PROGRAM commute8                                          ! Start of source code
! This program reads commuting data from a file of census tract pairs and 
! sets up two direct access files for use by the AIRQUAL program. 
!
! Program developed by ICF Consulting for EPA in June 2000.
! Latest revision by MLG on June 1, 2000.
! Revised to change matching mechanism in June, 2002
! Revised for update to new 2000 commute file in July 12,2002
! Revised by MH in August 2005 to add travel time
! Added GETARG function that allows for parameters file to be called from command line by CC on August 27,2014
! Expanded wtractu, udispro, ngroup, uareapro, htractid, first2, and last2 dimensions to 99000 by CC on August 27,2014
! Commute7 finalized by ICF September, 25, 2014
!
! Files used
! 10 : param_file.1
!      PARAM file used for input constants & filenames
!      (also used by all other subprograms)
! 18 : log.base96.txt.usa
!      LOG file (also used by all other programs in the APPEND mode)
! 19 : s_allusa_comm.txt
!      Commute input ASCII data for the entire nation (used only here)
! 20 : Distance to Road Probability file (used only here)
! 29 : s_allusa_comm.da
!      Sequential file for all work tract ID used by HAPEM subprogram
! 39 : s_allusa_comm.ind
!      Sequential commute index file with population tractID used in HAPEM
! 44 : census_direct.ind
!      Direct access census tractID file generated by INDEXPOP
! 60 : mistract.dat
!      ASCII file showing population tracts not matched by home tracts
!      Only for book-keeping. Also used in AIRQUAL and HAEM subprograms
! 94 : s_allusa_comm.st_comm1_fip_range
!      Direct access file for indices of total # of population tracts per state
!      This file is used in HAPEM subprogram
! 96 : census.county_tract_pop_range
!      Direct access file for indices of total number of population tracts
!      for each county. This file is used in COMMUTE subprogram
! 97 : census.state_county_pop_range
!      Direct access file for indices of total number of population counties
!      per state. This file is used in COMMUTE subprogram
! 98 : statefip.dat
!      ASCII file for two digits state FIPs with their serial numbers
!      Also used by INDEXPOP and AIRQUAL subprograms
! 99 : counter.dat (This ASCII file is used in all subprograms)
!      This file has total number of chad entries in the input and output files
!
!  Declarations section
!
IMPLICIT NONE                                         ! Declare all variables
INTEGER, PARAMETER :: maxst = 53                      ! max # of states
INTEGER :: cl1, cl2, cr, cm                           ! Job timing variables
INTEGER :: keepfile=1								  ! If keepfile=1, imtermediate files
													  ! will be deleted
INTEGER :: ios, sum                                        ! Read/write indicator
INTEGER :: ngroup
INTEGER :: it,ni,n,nflow,nflow1,itt,ittt,cnum,i,ii,iii,ij,ist1,ist ! Indices and counters
INTEGER :: ntrac19,iflag,ntrac141,ntract,acount ! Number of commuters
INTEGER :: first,last,first1,last1,compair=0                   ! Index range for work tractid
INTEGER :: htractno									  ! counter of htracts in com file
INTEGER :: varint,stnum,stsuma,stsump
INTEGER :: region1, region2                           ! Serial # of states be included
INTEGER :: sta(maxst),stp(maxst),stnum1(maxst)        ! Index counters
INTEGER :: bnum(maxst),enum(maxst),stnum2(maxst)      ! Index counters
INTEGER, ALLOCATABLE :: sta1(:), stp1(:)              ! Index counters
INTEGER, ALLOCATABLE :: first2(:), last2(:)			  ! indices of work tracts in com file
REAL :: cflow, cdist,cuflow,sflow,distcu                           ! cumulative fraction
REAL, ALLOCATABLE :: adist(:),dist(:)			      ! Distance travelled
REAL, ALLOCATABLE :: flow(:),aflow(:)				  ! Cumulative fractions of people commuted to work tracts
REAL, ALLOCATABLE :: rflow(:)                         ! fraction of people commuted per work tract
CHARACTER (LEN=1) :: cf
CHARACTER (LEN=2) :: stind(maxst), astind(maxst)
CHARACTER (LEN=2) :: stfip                            ! state fip in 2 strings
CHARACTER (LEN=100) :: f10='',f14,f18,f19,f20,f30,f40,f29,f39,f44   ! File names
CHARACTER (LEN=100) :: f60,f94,f96,f97,f98,f99,f201,f202,f301,f302,f401,f402              ! File names
CHARACTER (LEN=120) :: paramline,varname,varchar,FNAME            ! PARAMS file input
CHARACTER (LEN=11)  :: htractid2,htractid1
CHARACTER (LEN=11), ALLOCATABLE :: htractid(:), wtractid(:) ! home & work tractid
CHARACTER (LEN=11), ALLOCATABLE :: ahtractid(:), awtractid(:) ! home & work tractid
CHARACTER (LEN=11), ALLOCATABLE ::  ctractid(:) ! population  tractids 
CHARACTER (LEN=11), ALLOCATABLE :: county(:)

! New Variables

CHARACTER (LEN=2), ALLOCATABLE :: stateidd(:), stateidm(:), stateidf(:)
CHARACTER (LEN=11) :: htract,wtract,distract,comtract, fractract,tractid001,wtractu(99000)
!CHARACTER (LEN=11) :: htract,wtract,distract,comtract, fractract,tractid001,wtractu(66500)
INTEGER, ALLOCATABLE :: begdist(:),enddist(:), begcomt(:),endcomt(:),begfrac(:),endfrac(:)
INTEGER :: ndstate,j,k,idsel,find14,nfind,findx,find15
INTEGER :: mdstate,idselm,findm
INTEGER :: fdstate,idself,findf
INTEGER :: ik,ik2,ik3
REAL :: flowx,distx !dispro2(3),Udispro(3,99000)
!REAL :: flowx,distx !dispro2(3),Udispro(3,66500)
REAL :: comtimetp(2), comtime(2),compro(2),comprotp(2)
!REAL :: fracpro(2),fracprotp(2)
REAL, allocatable :: dispro(:,:),areapro(:),disprotp(:,:),areaprotp(:),Udispro(:,:,:),Uareapro(:,:),dispro2(:,:),areapro2(:)
REAL,allocatable :: fracpro(:,:),fracprotp(:,:)
!
!  PARAMS section
!
CALL SYSTEM_CLOCK(cl1,cr,cm)                          ! Execution start time
CALL GETARG(1,FNAME)
OPEN (UNIT=10,FILE=FNAME,RECL=120,ACTION='READ')            
!
DO                                                    ! Loop over records
   READ(10,'(A120)',IOSTAT=ios) paramline             ! Read one PARAM line
   IF (ios/=0) EXIT                                   ! Exit on end of file
   i =INDEX(paramline,'=')                            ! Check for equals sign
   IF (i==0) CYCLE                                    ! Discard line if no '='
   varname=ADJUSTL(paramline(1:i-1))                  ! Part on left of '='
   varchar=ADJUSTL(paramline(i+1:))                   ! Part on right of '='
   IF (INDEX(varname,'keep')>0) cf=varchar(1:1)       ! Read commuting flag 
   IF ((cf=='Y').OR.(cf=='y')) keepfile=2             ! Set commuting variable
   IF (INDEX(varname,'file')>0) THEN                  ! For file names...
      IF (INDEX(varname,'populat')>0)  f14=varchar    ! Set popultion file name
      IF (INDEX(varname,'log')>0)      f18=varchar    ! Set log file name
      IF (INDEX(varname,'commut')>0)   f19=varchar    ! Set commuting file name
      IF (INDEX(varname,'mistract')>0)  f60=varchar   ! Set mistract file name
      IF (INDEX(varname,'counter')>0)   f99=varchar   ! Set counter file name
      IF (INDEX(varname,'statefip')>0)  f98=varchar   ! Set statefip file name
	  IF (INDEX(varname,'DistToRoad')>0) f20=varchar  ! Set the distance file name
	  IF (INDEX(varname,'CommutTime')>0) f30=varchar  ! Set the commute time file
	  IF (INDEX(varname,'CommutFrac')>0) f40=varchar  ! Set the commute frac file
   else
      READ(varchar,*,IOSTAT=ios) varint
      IF (ios/=0) CYCLE
      IF (varname=='region1')        region1=varint   ! first state for run
      IF (varname=='region2')        region2=varint   ! last state for rn
      IF (varname=='ngroup')         ngroup =varint ! Set # demographic groups
	  IF (varname=='nreplic') then
	     write(*,*) 'Wrong parameter file.  Use P1.txt for this module'
		 stop
	   end if 
	  END IF                                             ! Finished with line
END DO                                                ! Get next line
CLOSE(10)                                             ! Close PARAMS file
!
!  Setup section
!
i=INDEX(f19,'.')-1                                    ! Locate extension on f19
IF (i==0) i=LEN_TRIM(f19)                             ! Trim f19 file name
f29=f19(:i)//'.da'                                    ! Direct access file name
f39=f19(:i)//'.ind'                                   ! Index file name 
f94=f19(:i)//'st_comm1_fip_range'
i=INDEX(f14,'.')-1                                    ! Locate extension on f19
IF (i==0) i=LEN_TRIM(f19)                             ! Trim f19 file name
f44=f14(:i)//'_direct.ind'                            ! Direct access file name
f96=f14(:i)//'.county_tract_pop_range'
f97=f14(:i)//'.state_county_pop_range'

i=INDEX(f20,'.')-1
IF (i==0) i=LEN_TRIM(f20)
f201=f20(:i)//'.STIDX'
f202=f20(:i)//'.dat'
!
i=INDEX(f30,'.')-1
IF (i==0) i=LEN_TRIM(f20)
f301=f30(:i)//'.STIDX'
f302=f30(:i)//'.dat'

i=INDEX(f40,'.')-1
IF (i==0) i=LEN_TRIM(f20)
f401=f40(:i)//'.STIDX'
f402=f40(:i)//'.dat'


Allocate (dispro(3,ngroup),disprotp(3,ngroup),areapro(3),areaprotp(3),dispro2(3,ngroup),areapro2(3),Udispro(3,ngroup,99000),Uareapro(3,99000))
Allocate (fracpro(2,ngroup),fracprotp(2,ngroup))

OPEN (18,FILE=f18,POSITION='APPEND',ACTION='WRITE')   ! Open log file
OPEN (19,FILE=f19,ACTION='READ')                      ! Open commute input file
OPEN (201,FILE=f201,ACTION='read')
OPEN (202,FILE=f202,ACCESS='direct',RECL=15+3*ngroup*4,ACTION='read')
!OPEN (20,FILE=f20,ACTION='READ')					   ! Open distance probability file
!OPEN (29,FILE=f29,ACCESS='DIRECT',RECL=50)            ! Open commute flow file
OPEN (301,FILE=f301,ACTION='read')
OPEN (302,FILE=f302,ACCESS='direct',RECL=15+5*4,ACTION='read')
OPEN (401,FILE=f401,ACTION='read')
OPEN (402,FILE=f402,ACCESS='direct',RECL=15+2*ngroup*4,ACTION='read')

!OPEN (291,FILE='temp1.txt',ACTION='READWRITE',status='scratch')            ! Open commute flow file
OPEN (291,FILE='temp1.txt',ACTION='READWRITE') 
!OPEN (292,FILE='temp3.txt',ACTION='READWRITE')
!OPEN (39,FILE=f39,ACCESS='DIRECT',RECL=50)            ! Open commute index file
!OPEN (391,FILE='temp2.txt',ACTION='READWRITE',status='scratch')
OPEN (391,FILE='temp2.txt',ACTION='READWRITE')
OPEN (29,FILE=f29,ACTION='write')            ! Open commute flow file
OPEN (39,FILE=f39,ACTION='write') 
OPEN (44,FILE=f44,ACCESS='DIRECT',RECL=30)            ! Open population tractid file
open (60,file=f60, position='rewind', action='write') ! mistract file
!OPEN (94,FILE=f94, ACCESS = 'direct',recl=50)         ! Index file for state commute
OPEN (94,FILE=f94,ACTION='WRITE')
!OPEN (96,FILE=f96, ACCESS='DIRECT',RECL=70)           ! census indices file
!OPEN (97,FILE=f97, ACCESS='DIRECT',RECL=70)           ! census indices file
OPEN (96,FILE=f96, ACTION='READ')           ! census indices file
OPEN (97,FILE=f97, ACTION='READ')           ! census indices file
OPEN (98,FILE=f98, action='read')                     ! state fip file
OPEN (99,FILE=f99, status='old')                      ! counter file
!
WRITE(18,*) 'Commute Start time= ',cl1,cr,cm          ! Execution start time
!

ndstate=0

DO 

read(201,*,iostat=ios)
if(ios/=0) exit
ndstate=ndstate+1

END DO 

Allocate(stateidd(ndstate),begdist(ndstate), enddist(ndstate))

rewind(201)

Do i=1, ndstate

read (201,*) k,stateidd(i), begdist(i), enddist(i)

end do 

! commuting time indexing file

mdstate=0

DO 

read(301,*,iostat=ios)
if(ios/=0) exit
mdstate=mdstate+1

END DO 

Allocate(stateidm(mdstate),begcomt(mdstate), endcomt(mdstate))

rewind(301)

Do i=1, mdstate

	read (301,*) k,stateidm(i), begcomt(i), endcomt(i)

end do 
!
! begin of commut frac file

fdstate=0

DO 

read(401,*,iostat=ios)
if(ios/=0) exit
fdstate=fdstate+1

END DO 

Allocate(stateidf(fdstate),begfrac(fdstate), endfrac(fdstate))

rewind(401)

Do i=1, fdstate

	read (401,*) k,stateidf(i), begfrac(i), endfrac(i)

end do 
!

! read two digit state fips
!
do ii = 1, maxst
  read (98, 81) ij, stind(ii)
end do
81 format (i2, 6x, a, 6x, i2)
!
!  Indexing section
!
!count total number of tracts in commute file for memory allocation
!
ntrac19 = 0
DO                        
   READ (19,*,IOSTAT=ios) 
   IF (ios/=0) EXIT
   ntrac19= ntrac19 + 1
END DO
REWIND(19)
!
!Assign memory
!

ALLOCATE (htractid(99000), first2(99000),last2(99000),wtractid(ntrac19),dist(ntrac19), flow(ntrac19))
ALLOCATE (ahtractid(ntrac19),awtractid(ntrac19),adist(ntrac19),aflow(ntrac19),rflow(ntrac19))
!
!Find total number of commuting tracts per state
!
enum = 0
DO it = 1, ntrac19
!   READ (19,*,IOSTAT=ios) htractid2, wtractid(it), dist(it), flow(it), cuflow
   READ (19,*,IOSTAT=ios) htractid2, wtractid(it), dist(it), flow(it)
!   write (*,*) htractid2, wtractid(it), dist(it), flow(it)
END DO
!
REWIND(19)
first1=0
last1=0
htractno=0

DO 
	READ (19,*,IOSTAT=ios) htractid1
	if (ios/=0) exit
	htractno=htractno+1
	first1=last1+1
	n=1
	DO
		
		READ (19,*,IOSTAT=ios) htractid2
		if (ios/=0) exit	
		if(htractid2 ==htractid1) then
			n=n+1
		else
			backspace(19)
			exit
		end if
	end do
	last1=last1+n
	htractid(htractno)=htractid1
	first2(htractno)=first1
	last2(htractno)=last1
    
	do ii = 1, maxst
     if (stind(ii) == htractid(htractno)(1:2)) then
       enum(ii) = enum(ii) + 1
       exit
     end if
   end do

END DO

!write(*,*) ntrac19, last1,htractno

! Assign the indices range for commuting home tracts for each state
!
stnum1 = 0
stnum2 = 0
stnum = 0
do ii = 1, maxst
  stnum1(ii) = stnum + 1
  stnum2(ii) = stnum1(ii) + enum(ii)-1
  stnum = stnum2(ii)
end do
!
! read total number of tracts in population file and assign memory
!
read (99,*)
read (99,*) ntract, acount
ALLOCATE (ctractid(ntract),county(acount),stp1(acount),sta1(acount))
!
!read total number of counties per state
!
do ii = 1, maxst
  read (97,*,IOSTAT=ios) astind(ii),stp(ii),sta(ii)
!  write(*,*) astind(ii),stp(ii),sta(ii), ii
end do
!
!read indices range of total number of population tracts per state county
!
do ii = 1, acount
  read (96,*,IOSTAT=ios) county(ii), stp1(ii), sta1(ii)
!  write (*,*) county(ii), stp1(ii), sta1(ii)
end do
!
!read tractid of all population tracts
!
do ii = 1, ntract
  read (44,rec=ii) ctractid(ii)
end do
!
cnum  = 0
first = 0
last  = 0
bnum  = 0
ntrac141 = 0
!
1411 format('# of Home Tracts:', I7,'  # of Work Tracts:',I7)
1412 format('# of Home/Work Tract Pairs:', I8)
!
write (60,*) 'MISSING TRACTS OF COMMUTE & AIRQUAL in COMMUTE'

! DEBUGGING FILES
!OPEN (67,FILE="LoopCheck.txt",POSITION='REWIND',ACTION='WRITE') ! Writes State loop, county, and census track loop
!OPEN (68,FILE="418Check.txt",POSITION='REWIND',ACTION='WRITE')
!OPEN (69,FILE="ctractIdCheck.txt",POSITION='REWIND',ACTION='WRITE')
!OPEN (699,FILE="291Copy.txt",POSITION='REWIND',ACTION='WRITE')
!
do ist = region1, region2                               ! state loop
  do ist1 = stp(ist), sta(ist)                          ! county loop
    do it = stp1(ist1), sta1(ist1)                      ! census tract loop
!	  write (67, *) ist, ist1, it ! DEBUGGING CODE
      ntrac141 = ntrac141+1
      cnum = cnum + 1
!	  write(*,1411) ntrac141
	  first = last + 1
      sflow = 0
	  distcu=0
      nflow = 0
	  nflow1=0
      iflag = 0
      do itt = stnum1(ist), stnum2(ist)              ! commute home tract/state loop
        if (htractid(itt) == '  ') cycle             
        if (htractid(itt)==ctractid(it)) then        ! home tract and census tract matched
!		  write(69, *) htractid(itt), ctractid(it)
          iflag = 1
		  DO ittt=first2(itt),last2(itt)
!		    nflow1=nflow1+1
			if (dist(ittt) <= 120.0) then               ! commuting distance limit
				sflow = sflow + flow(ittt)
				distcu=distcu + dist(ittt)         
				nflow = nflow + 1                        ! counter increment
!				Write(688,*) ist, ist1, it, nflow
!				write(*,*) nflow1,nflow,flow(ittt),sflow
				awtractid(nflow) = wtractid(ittt)
				adist(nflow) = dist(ittt)
				aflow(nflow) = flow(ittt)
			end if
		  end do 
          htractid(itt) = '  '                       ! avoid duplication
		  exit
        end if
      END DO
!
      cflow = 0.0
	  cdist = 0.0                                    ! cimulative fraction per census tract
      if (sflow /= 0) then                           ! total people commuting
        DO ni = 1, nflow                             ! for all worktracts
          last = last + 1
          rflow(ni) = aflow(ni)/sflow
!		  Write (68, *) ist, ist1, it, ni, aflow(ni), sflow ! DEBUGGING STATEMENT
          cflow = cflow + rflow(ni)
		  
		  if(distcu/=0) then						 ! calculate the dist/average
			cdist = adist(ni)/distcu*real(nflow)     
		  else if (distcu==0) then
			cdist = 0.0
		  end if
          
		  if(ni== nflow .and. cflow/=1.0) cflow=1.0   ! remove 0.999999 due to calculation error
		  
!		  WRITE(699,*) awtractid(ni),cflow, cdist
		  WRITE(291,*,IOSTAT=ios) awtractid(ni),cflow, cdist ! write work tractid and corresponding cumulative fractions
        end do 
      else if (sflow == 0) then                      ! no body is commuting
        if (last < first) last = last + 1
        WRITE(291,*,IOSTAT=ios) ctractid(it), 1.0, 0.0  ! Let everyone stay home and travel distance zero
      END IF                                         ! Done with this home
      if (ctractid(it)(1:2) == stind(ist)) then      ! match with statefip   
        bnum(ist) = bnum(ist) + 1                    ! counter increment
      end if
      if (iflag == 0) write (60,104) it,ist1,ist,ctractid(it),iflag
!      WRITE(39,REC=cnum) ctractid(it), first, last,iflag
		WRITE(391,*,IOSTAT=ios) ctractid(it),first,last,iflag
!	   WRITE(*,1411) ntrac141,last
    END DO                                           ! Next census tract 
  end do                                             ! next county   
end do                                               ! next state  

! CLOSING DEBUGGING FILES
!Close (67)
!Close (68)
!Close (69) 
!CLOSE (688)
!CLOSE (699)
104 format(3(1x,i8),1x,a,1x,i2)
compair=last
!WRITE(*,1411) ntrac141,last
write (99,*) ntrac141, ntrac19
write (99, *) last, cnum
!
!write indices for total number of population tracts per state
!
stsuma = 0
stsump = 0
do ii = 1, maxst
   stsuma = stsump + bnum(ii)
   write (94,*,IOSTAT=ios) stind(ii), stsump+1, stsuma
!  write (52,80) ii, stind(ii), stsump+1, stsuma, bnum(ii)
   stsump = stsuma
end do
!WRITE (*,*) 'Reached Line 480'
!
80 format (2x,i10,2x,a,2x,i10,2x,i10,2x,i10)
!

! new code for distance probability

rewind(291)
rewind(391)
3911 format(A12,2I8,I3,2F8.4,2F10.4,60F8.4)
DO 

	read (391,*,iostat=ios) htract,first,last,iflag
	if(ios/=0) exit
	idsel=0
	idselm=0
	idself=0
	findx=0
	findm=0
	findf=0
	do i=1, ndstate
		if(htract(1:2) == stateidd(i)) then
		idsel=i
		exit
		end if		
	end do
	
	do i=1, mdstate
		if(htract(1:2) == stateidm(i)) then
		idselm=i
		exit
		end if		
	end do

	do i=1, fdstate
		if(htract(1:2) == stateidf(i)) then
		idself=i
		exit
		end if		
	end do	 

	if(idsel>0) then
	disprotp=0.0
	areaprotp=0.0
	do j=begdist(idsel),enddist(idsel)
		read(202,rec=j) distract,areapro(:),(dispro(:,ik),ik=1,ngroup)
!		write(*,*) distract
!		write(*,*) htract
!		write(*,*) begdist(idsel),' ',enddist(idsel),' ',idsel,' ',j
!		stop
		if(htract==distract) then
		disprotp=dispro
		areaprotp=areapro
		findx=1
		exit
		end if
	end do
	else if (idsel==0) then
		write(*,*) 'Error: no matched record found in DistToRoad file'
		stop
	end if
	
	if(idselm>0) then
	comtimetp =0.0
	do j=begcomt(idselm),endcomt(idselm)
		read(302,rec=j) comtract,compro(:),comtime(:)
!		write(*,*) distract
!		write(*,*) htract
!		write(*,*) begdist(idsel),' ',enddist(idsel),' ',idsel,' ',j
!		stop
		if(htract==comtract) then
		comtimetp=comtime
		comprotp=compro
		findm=1
		exit
		end if
	end do
	else if (idselm==0) then
		write(*,*) 'Error: no matched record found in CommutTime file'
		stop
	end if	

	if(idself>0) then
	fracprotp =0.0
	do j=begfrac(idself),endfrac(idself)
		read(402,rec=j) fractract,(fracpro(:,ik),ik=1,ngroup)
!		write(*,*) distract
!		write(*,*) htract
!		write(*,*) begdist(idsel),' ',enddist(idsel),' ',idsel,' ',j
!		stop
		if(htract==fractract) then
		fracprotp=fracpro
		findf=1
		exit
		end if
	end do
	else if (idself==0) then
		write(*,*) 'Error: no matched record found in CommutFrac file'
		stop
	end if

	
	if (findx==1 .and. findm==1 .and. findf==1) then
		write(39,3911) htract,first,last,iflag, comprotp(:),comtimetp(:),areaprotp(:),(disprotp(:,ik), ik=1,ngroup), (fracprotp(:,ik2),ik2=1,ngroup)

	else 
		write(*,*) 'error with Distance Probability or CommutTime or CommutFrac files'
		stop
	end if

end do 

!WRITE (*,*) 'Reached Line 592'
nfind=0

! DEBUGGING STATEMENT
!OPEN (597,FILE="600.txt",POSITION='REWIND',ACTION='WRITE')
Do 

	read(291,*,iostat=ios) tractid001
	if(ios/=0) exit
	find14=1
	Do i=1,nfind
		if(tractid001==wtractu(i)) then
			find14=0
			exit
		end if
	end do
	if(find14==1) then
			nfind=nfind+1
			wtractu(nfind)=tractid001
	end if
!	write(597,*) tractid001, wtractu(nfind) ! DEBUGGING STATEMENT
end do

!Close (597)
!WRITE (*,*) 'Reached Line 613'
!WRITE (*,*) nfind
!write(*,1412) nfind
rewind(291)

2921 format(A12,3F8.4)

!OPEN (620,FILE="620.txt",POSITION='REWIND',ACTION='WRITE')
Do iii=1, nfind
! iii=mod function
	idsel=0
	findx=0
	do i=1, ndstate
		if(wtractu(iii)(1:2) == stateidd(i)) then
		idsel=i
		if (iii==1) then
!			write(*,*) 'Works'
		end if
		exit
		end if		
	end do
	if(idsel>0) then
	do j=begdist(idsel),enddist(idsel)
		read(202,rec=j) distract,areapro(:),(dispro(:,ik),ik=1,ngroup)
!		Write (620,*) iii, j, wtractu(iii), distract, areapro(:),(dispro(:,ik),ik=1,ngroup)
		if(wtractu(iii)==distract) then
		Udispro(:,:,iii)=dispro(:,:)
		Uareapro(:,iii)=areapro(:)
		findx=1
		exit
		end if
	end do
	else if (idsel==0) then
		write(*,*) 'error with files'
		stop
	end if 
		
	if(findx==0) then
		write(*,*) 'error with files'
		stop
	end if
end do

!Close(620)
!WRITE (*,*) 'Reached Line 650'
291 format(A12, F11.7,33F8.4, F8.4)

Do 

	read(291,*,iostat=ios) wtract, flowx, distx
	if(ios/=0) exit
	dispro2=0
	areapro2=0
	find15=0
	do i=1, nfind
		if(wtractu(i)==wtract) then
		dispro2(:,:)=Udispro(:,:,i)
		areapro2(:)=Uareapro(:,i)
		find15=1
		exit
		end if
	end do 			
	
		if(find15==1) then

		write(29,291) wtract,flowx, areapro2(:),(dispro2(:,ik),ik=1,ngroup),distx

		else if(find15==0) then

		write(*,*) 'error with distance_probability file'
		stop

		end if 


end do 

!WRITE (*,*) 'Reached Line 683'
WRITE(*,1411) ntrac141,nfind
WRITE(*,1412) compair
!  Wrapup section
!
CLOSE(19)                                           ! Close tract pair file
CLOSE(29)                                           ! Close data file
CLOSE(39)                                           ! Close index file 
close(60)
close(94)

if(keepfile==1) then
close(96, status='delete')
close(97, status='delete')
else
close(96)
close(97)
end if
deallocate (dispro,disprotp,dispro2,Udispro,Uareapro,areapro,areaprotp,areapro2)
deallocate (fracpro,fracprotp)
close(98)
close(99)
CALL SYSTEM_CLOCK(cl2,cr,cm)                        ! Get execution stop time
WRITE(18,*) 'Commute End time = ',cl2               ! Write time to log
WRITE(18,*) 'Commute Job time = ',REAL(cl2-cl1)/cr  ! Total time for job
CLOSE(18)                                           ! Close log file


!WRITE (*,*) 'Reached End of Code' 
STOP                                                ! Stop execution
END PROGRAM commute8                                ! Last line of source code

Subroutine LoadData(f20)

end subroutine 
