PROGRAM hapem8                                            ! Start of source code
! This program carries out the exposure calculation in the HAPEM model.
! It reads job parameters from the PARAMS file as unit 10.
! It has capability to process the distribution data for micro-environmental factors
!
! Program developed by ICF Consulting for EPA in June 2000.
! Revised by MH and ASR on August 2, 2001
! Revised by MH and ASR on March 31, 2002 
! Revised by MH and ASR on May 15, 2002
! Revised for implementing random selection of factors and air data sets
! by MH on June 5, 2002
! Revised for adding variable background concentration on July 10,2002
! Revised by MH on November 18, 2003 for adding random selection for ADD factor
! Revised by MH on November 19, 2003 for adding random selection for Combustion ADD factor and separate output
! Revised by MH on Nov. 23, 2003 for adding residential garage penetration factor
! Revised by MH on Nov. 24, 2003 for adding the forth indoor source-Residence with garages
! Finalized by MH on Dec.5, 2003
! Revised by MH on Dec.30 for moving call random2 outside the hblock loops in the ADD factor section 
! Files used
! Revised by MH on Jan 15,2004 for error message in the DOS screen
! Revised by MH on May 17,2005 for adding a variable number of source categories
! Revised by MH on August 8, 2005 for hapem6
! Revised by Casey Cavanagh on August 15, 2014 for hapem6_2014
! New updates adds in capability to read file name from cmd (Line 225)
! and increases array dimension to handle larger files (Line 102)
! Hapem7 finalized by ICF September, 25, 2014
! Revised by WGG in August-October 2023
! Problems with AREAW=0 fixed by setting to small positive number
! Hapem8 finalized by ICF on 4 October 2023  

!
! 10 : param_file.2
!      PARAM file used for input constants & filenames
!      (also used by airqual5)
! 15 : factors.txt
!      Factor distribution file (ASCII) for specific pollutant
! 18 : log_file.sarod
!      LOG file (also used by all other programs in the APPEND mode)
! 21 : durhw.da (Final CHAD output file generated by DURAV)
!      Sequential file with final activity data grouped by 
!      day and group type combinations 
! 23 : benzout.da
!      Sequential air quality file created by airqual and used in this program
! 29 : s_allusa_comm.da
!      Sequential file for all work tract ID created by COMMUTE subprogram
! 30 : durhw.nonzero (This ASCII file is created by DURAV)
!      This file has number of CHAD entries per group and day types and also 
!      flgas indicating whether that combination of group/day type corresponds
!      to commute or not.
! 31 : clustertransa.txt
!	   This file has the frequency of clusters in demo group/daytype and probabilities
!	   of transition among clusters.  The frequency and probabilities are represented 
!	   in accumulative fractions.
! 35 : census.pop_air_da (This file is created by AIRQUAL)
!      Sequential file for population data of tracts matched with airtracts
! 39 : s_allusa_comm.ind (this file is created by COMMUTE)
!      Sequential commute index file with population tractID.
! 43 : benzout.air_da (This file is created by AIRQUAL)
!      Sequential file tracts matched with pop tracts and indices ranges 
!	   for air quality records in file 23. Created by Airqual and used in this 
!	   program
! 60 : mistract.dat
!      ASCII file showing population tracts not matched by home tracts
!      Only for book-keeping. Also used in AIRQUAL and HAPEM subprograms
! 61 : 
!      Final exposure output file for each state
!      Will only contain the output for total number of airtracts less than or
!      equal to 31000
! 92 : benzout.state_air1_fip_range
!      Sequential file for indices of # of airquality counties per state
!      This file is cre
! 93 : benzout.state_air_fip_range
!      Sequential file for indices of # of airquality tracts per county
!      This file is used in HAPEM subprogram
! 94 : if commuting is included:
!      s_allusa_comm.st_comm1_fip_range
!      otherwise
!	   benzout.state_air2_fip_range (FILE 91 created by airqual5)
!      Sequential file for indices of total # of population tracts per state,
!      either from commuting tracts or air quality tracts
! 99 : counter.dat (This ASCII file is used in all subprograms)
!      This file has total number of chad entries in the input and output files
!
!  Declarations section
!
IMPLICIT NONE                           ! Declare all variables
INTEGER, PARAMETER :: maxst = 53        ! Maximum number of states or state-like regions
INTEGER            :: amax = 30			! max # of replicates for each tract
INTEGER, PARAMETER :: fracmax=2         ! max # of population fractions 
INTEGER, PARAMETER :: maxFNO =12		! max # of columns used for factor distribution data
INTEGER, PARAMETER :: maxdist=3	        ! max # of sets of distribution for mobile sources
INTEGER, PARAMETER :: maxring=3			! # of near road distance rings
INTEGER, PARAMETER :: maxNo	= 500		! max # of air data sets in a tract
INTEGER, PARAMETER :: maxcls=3			! max # of cluster in group/day combinations
INTEGER, PARAMETER :: maxProd = 80		! max # of products/materials/combustion sources in indoor conc. files
INTEGER, PARAMETER :: maxauto = 50		! max # of auto/chemical products used in garages
INTEGER, PARAMETER :: ncom = 2			! # of commuting categories
INTEGER, PARAMETER :: ncar = 2
!
INTEGER :: stp(maxst),sta(maxst),sumc,ist,sflow,td
INTEGER :: idx20
INTEGER :: stsump(maxst),stsuma(maxst),region1,region2,isum=0,cnum
INTEGER :: itest, itract, ntrac14=0, ntrac19, atract, ifip, ktest
INTEGER :: jc,ic,kc,kcdumy,mc,ii,jj,ifnd,ifnd1,ifnd2,ifind,icom
INTEGER :: commut=1, max1= 228000                        ! Commuting flag (1=NO, 2=YES)
INTEGER :: year, wtract, jtest = 0, acount
INTEGER :: isam, nsel=100, idum = -1,idum2=-1,idum21,idum3=-1,idum4=-1,autoY=1,autofind=1,indauto=0  !autoY=1 means NO
INTEGER :: idummb1=-1
INTEGER :: ISP1P2 = 0,EPAreg									! indicates if the right parameter is used
INTEGER :: isam2, isum2, t2, g2
INTEGER :: m, b, s, k1,g, t, i, k,j,j1,c, n, ib,s1,s2,s3,s4     ! Loop indices
INTEGER :: ip1,ip2,ip3,ierror1,ierror2,ierror3,ierror4,ierror5,ierror6,ierror7,ierror8,ierror9
INTEGER :: id1,id2,id3									! index for product/material1or2 indoor conc files
INTEGER :: npt,nmt1,nmt2,nct,nat
INTEGER :: ipt,imt,ict,itt,ProSel,ki
INTEGER :: n11, i3,i2,i1,h1
INTEGER :: cl1, cl2, cl3, cm, cr                       ! Job timing variables
INTEGER :: yproduct=1,ym1=1,ym2=1,ycomb=1,yauto=1
INTEGER :: varint, ios, status1                        ! Temporary variables
INTEGER :: nmicro,hblock,ntype,ngroup,nsource   ! Upper bounds for loops
INTEGER :: AirNo,AirNo2,AirSel							! Number of Air Datasets 
INTEGER :: micron,atract2, airecord,arecord
INTEGER :: penA1, addA1, proxA1,days1
INTEGER :: igroup, iday, cls,cls2,clsno,clusno1,icluster	! # of demo group, day type, and cluster
INTEGER :: dcom,dcom2,icomm
INTEGER, ALLOCATABLE :: lag(:,:,:)                ! Indicator for one-hour lags
INTEGER, ALLOCATABLE :: pop(:,:)                ! Population of demo groups
INTEGER, ALLOCATABLE :: durs(:,:)
INTEGER, ALLOCATABLE :: ival(:,:,:,:),jcom(:,:),clusno(:,:,:),randcls(:,:)
INTEGER, ALLOCATABLE :: psump(:), asuma(:)
INTEGER, ALLOCATABLE :: first(:), last(:), first1(:),last1(:), iflag(:), rand (:)
INTEGER, ALLOCATABLE :: n1(:)				 ! number of airqual records in the same tracts
INTEGER, ALLOCATABLE :: clsdays(:,:),clsdays2(:),clsdays1(:)
INTEGER, ALLOCATABLE :: zoneidxh(:,:),zoneidxw(:,:)
INTEGER :: nmobiles(10)=0, nmobiles2(10)=0,nmsource,nmsource2,nmrecord
INTEGER :: itq, itq1,findnm,findist,findsr
INTEGER :: nbmicro, nemicro, nRD2
INTEGER :: nvehicles(10)=0,nmvehicle
INTEGER :: itq2,findv
INTEGER :: dcomSel,dcar,dcarSel
INTEGER :: commutnew
INTEGER :: ik
INTEGER :: npublict(10)=0,nmpublict
INTEGER :: itq3
INTEGER :: findcax
!
REAL :: arand,arand4,arand5,arand2,arand3,dexp1,dexp2,total,tday,bconc,dexpb,dexpaP,dexpaM,dexpaC,dexpaT,avarint
REAL :: penA2,penA3,penA4,penL,penU, addA2,addA3, addA4, addL,addU,proxA2, proxA3, proxA4
REAL :: penX,addX,proxX, proxL,proxU
REAL :: amback2, ambackw2
REAL,    ALLOCATABLE :: frworkers(:), bmax(:), concb(:,:),concaddP(:,:),concaddM(:,:),concaddC(:,:),concaddT(:,:)
REAL,    ALLOCATABLE :: ambh(:,:,:),ambw(:,:,:),amback(:),ambackw(:)
REAL,    ALLOCATABLE :: ambh2(:,:),ambw2(:,:)    ! Ambient air quality array
REAL,    ALLOCATABLE :: amblh(:,:)
REAL,    ALLOCATABLE :: amblw(:,:)              ! One-hour lagged air quality
REAL,    ALLOCATABLE :: prox(:,:), proxA(:,:,:,:)               ! Multiplicative micro. factors
REAL,    ALLOCATABLE :: addP(:,:), addM(:,:), addC(:,:),addT(:,:),addA(:,:)    ! Additive micro. factors
REAL,    ALLOCATABLE :: pen(:,:),penA(:,:)     ! Unit conversion factors
REAL,    ALLOCATABLE :: autopen(:,:)				! Garage penetration factor
REAL,    ALLOCATABLE :: dur(:,:,:,:,:,:,:),durw(:,:)            ! Activity duration array
REAL,    ALLOCATABLE :: dursc(:,:,:),dursc2(:,:)			! Average Activity duration array
REAL,    ALLOCATABLE :: conc(:,:,:)               ! Microenvironmental conc
REAL,    ALLOCATABLE :: tconc(:,:), popf(:), aqout(:,:,:),backC(:)
REAL,    ALLOCATABLE :: cexp(:,:,:),fact(:),fexp(:,:),fexpaP(:,:),fexpaM(:,:),fexpaC(:,:),fexpaT(:,:)    ! Daily exposure array
REAL,    ALLOCATABLE :: TempArray(:,:,:,:), lagb(:), RandomOut(:,:,:),DataArray(:)
REAL,    ALLOCATABLE :: Freq(:,:,:,:), Prob(:,:,:,:,:),Prob1(:,:,:,:),Freq1(:),Prob11(:,:)
REAL,    ALLOCATABLE :: prodfra(:,:),m1fra(:,:),m2fra(:,:),combfra(:,:)
REAL,    ALLOCATABLE :: prodfra2(:,:),m1fra2(:,:),m2fra2(:,:),combfra2(:,:)
REAL,    ALLOCATABLE :: prodconc(:,:,:,:),m1conc(:,:,:,:),m2conc(:,:,:,:),combconc(:,:,:,:),autoconc(:,:,:,:)
REAL,    ALLOCATABLE :: prodconc2(:,:,:,:),m1conc2(:,:,:,:),m2conc2(:,:,:,:),combconc2(:,:,:,:),autoconc2(:,:,:,:)
REAL,    ALLOCATABLE :: DISTW(:,:,:), DISTH(:,:,:), DISTHA(:,:,:),AREAW(:,:),AREAH(:,:),AREAHA(:,:),RD2(:), COMPRO(:,:),COMTIME(:,:), DISTFRA(:),fracpop(:,:,:)
REAL :: DISTHTOT,DISTWTOT, RD2Q, Cterm,Afar,A2
REAL :: nvtotal,nrtotal,nptotal
REAL :: dcomtotal,dcartotal
REAL :: blocksum
!
CHARACTER (LEN=11), ALLOCATABLE :: wtractid(:),productid(:),material1(:),material2(:),combid(:),autoprod(:)
CHARACTER (LEN=2)   :: bstind(maxst), astind(maxst)
CHARACTER (LEN=5)   :: sarod
CHARACTER (LEN=100) :: path1
CHARACTER (LEN=999) :: pophead, commhead        ! Header records on pop. file
CHARACTER (LEN=120) :: paramline,varname,varchar, FNAME  ! PARAMS file input lines
CHARACTER (LEN=120) :: header                     ! Population file header
CHARACTER (LEN=120) :: M1,M2,M3,M4,M5,DF1,DF2,DF3,DF4,DF5 ! Long description of population demo groups
CHARACTER (LEN=100) :: f13,f11,f15,f18,f19,f21,f23,f29,f30,f31,f35,f39,f43 ! File names
CHARACTER (LEN=100) :: f60,f61,f90,f92,f93,f94,f99,f151,f152,f153,f1511,f1611 ! File names
CHARACTER (LEN=150) :: f1512,f1513,f1514,f1515,f1516,f1517,f1518,f1519,f1520,f15111
CHARACTER (LEN=40)  :: pollutant,units, EPAregion, CAS ! Formats and strings
CHARACTER (LEN=20)  :: head='     Pop     Average' ! String used on output file
CHARACTER (LEN=1)   :: cf='N'                   ! Commuting flag (character)
CHARACTER (LEN=6)   :: commbuf                  ! Buffer for commuting status
CHARACTER (LEN=11), ALLOCATABLE ::  atractid(:)
CHARACTER (LEN=5),  ALLOCATABLE :: ftractid(:)
CHARACTER (LEN=11), ALLOCATABLE :: ctractid(:)
CHARACTER (LEN=5)   :: gz
CHARACTER (LEN=100) :: f611
CHARACTER (LEN=8)   :: SOURCEID(20)

SOURCEID(1) = 'SOURCE01'
SOURCEID(2) = 'SOURCE02'
SOURCEID(3) = 'SOURCE03'
SOURCEID(4) = 'SOURCE04'
SOURCEID(5) = 'SOURCE05'
SOURCEID(6) = 'SOURCE06'
SOURCEID(7) = 'SOURCE07'
SOURCEID(8) = 'SOURCE08'
SOURCEID(9) = 'SOURCE09'
SOURCEID(10) = 'SOURCE10'
SOURCEID(11) = 'SOURCE11'
SOURCEID(12) = 'SOURCE12'
SOURCEID(13) = 'SOURCE13'
SOURCEID(14) = 'SOURCE14'
SOURCEID(15) = 'SOURCE15'
SOURCEID(16) = 'SOURCE16'
SOURCEID(17) = 'SOURCE17'
SOURCEID(18) = 'SOURCE18'
SOURCEID(19) = 'SOURCE19'
SOURCEID(20) = 'SOURCE20'


gz = 'gzip '
!
!   PARAMS section
!
CALL SYSTEM_CLOCK(cl1,cr,cm)                         ! Job execution start time
!

CALL GETARG(1,FNAME) !Grab the first command line argument 
 ! and store it in the variable 'FNAME' 

write(*,*) "Variable FNAME = ", trim(adjustl(FNAME)) 
OPEN (UNIT=10,FILE=FNAME,RECL=120,ACTION='READ') ! Open PARAMS file


!OPEN (67,FILE="durCheck.txt",POSITION='REWIND',ACTION='WRITE')  ! Open clst Test File
!
DO                                                   ! Loop over PARAMS records
   READ(10,'(A120)',IOSTAT=ios) paramline            ! Read one record
   IF (ios/=0) EXIT                                  ! Exit on end of file
   i =INDEX(paramline,'=')                           ! Check for equals sign
   IF (i==0) CYCLE                                   ! If no equals sign, cycle
   varname=ADJUSTL(paramline(1:i-1))                 ! Part on left of '='
   varchar=ADJUSTL(paramline(i+1:))                  ! Part on right of '='
   IF (INDEX(varname,'file')>0) THEN                 ! For file names...
      IF (INDEX(varname,'activity')>0) f11=varchar   ! Set time-activity file
      IF (INDEX(varname,'quality')>0)  f13=varchar   ! Set air quality file
      IF (INDEX(varname,'factors')>0)  f15=varchar   ! Set factors file name
	  IF (INDEX(varname,'product')>0)   f1511=varchar  ! Set add files pathname
	  IF (INDEX(varname,'AutoPduct')>0)f1611=varchar  ! Set auto garage product files
      IF (INDEX(varname,'log')>0)      f18=varchar   ! Set log file name
      IF (INDEX(varname,'commut')>0)   f19=varchar   ! Set commuting file name
	  IF (INDEX(varname,'ClusTrans')>0)f31=varchar   ! Set commuting file name
      IF (INDEX(varname,'populat')>0)  f35=varchar   ! Set population filename
      IF (INDEX(varname,'quality')>0)  f43=varchar   ! Set air quality file
      IF (INDEX(varname,'mistract')>0) f60=varchar   ! Set mistract file name
      IF (INDEX(varname,'afile')>0)  path1=varchar   ! Set output file name
      IF (INDEX(varname,'counter')>0)  f99=varchar   ! Set counter file name
	  IF (INDEX(varname,'mobiles')>0)  f15111=varchar  ! Set mobile factor name
   ELSE IF (INDEX(varname,'B_00')>0) THEN            ! For the pollutant name
      i = INDEX(varchar,'!')                         ! Look for comment
      IF (i>0) varchar=varchar(1:i-1)                ! Remove comment
      M1=varchar									 ! Set the pollutant name
   ELSE IF (INDEX(varname,'B_02')>0) THEN            ! For the pollutant name
      i = INDEX(varchar,'!')                         ! Look for comment
      IF (i>0) varchar=varchar(1:i-1)                ! Remove comment
      M2=varchar
   ELSE IF (INDEX(varname,'B_05')>0) THEN            ! For the pollutant name
      i = INDEX(varchar,'!')                         ! Look for comment
      IF (i>0) varchar=varchar(1:i-1)                ! Remove comment
      M3=varchar
   ELSE IF (INDEX(varname,'B_16')>0) THEN            ! For the pollutant name
      i = INDEX(varchar,'!')                         ! Look for comment
      IF (i>0) varchar=varchar(1:i-1)                ! Remove comment
      M4=varchar
   ELSE IF (INDEX(varname,'B_18')>0) THEN            ! For the pollutant name
      i = INDEX(varchar,'!')                         ! Look for comment
      IF (i>0) varchar=varchar(1:i-1)                ! Remove comment
      M5=varchar
   ELSE IF (INDEX(varname,'B_65')>0) THEN            ! For the pollutant name
      i = INDEX(varchar,'!')                         ! Look for comment
      IF (i>0) varchar=varchar(1:i-1)                ! Remove comment
      DF1=varchar									 ! Set the pollutant name
   ELSE IF (INDEX(varname,'poll')>0) THEN            ! For the pollutant name
      i = INDEX(varchar,'!')                         ! Look for comment
      IF (i>0) varchar=varchar(1:i-1)                ! Remove comment
      pollutant=varchar                              ! Set the pollutant name
   ELSE IF (INDEX(varname,'unit')>0) THEN            ! For the pollutant units
      i = INDEX(varchar,'!')                         ! Look for comment
      IF (i>0) varchar=varchar(1:i-1)                ! Remove comment
      units=varchar                                  ! Set the units
   ELSE IF (INDEX(varname,'CAS')>0) THEN             ! For the pollutant CAS
      i = INDEX(varchar,'!')                         ! Look for comment
      IF (i>0) varchar=varchar(1:i-1)                ! Remove comment
      CAS=varchar                                    ! Set the CAS
   ELSE IF (INDEX(varname,'EPA')>0) THEN             ! For setting EPA region
      i = INDEX(varchar,'!')                         ! Look for comment
      IF (i>0) varchar=varchar(1:i-1)                ! Remove comment
	  EPAregion = varchar                            ! Set the EPA Region     
   ELSE IF (INDEX(varname,'nmobiles')>0) THEN		 ! Specify the on-road sources
      i = INDEX(varchar,'!')						
	  IF (i>0) varchar=varchar(1:i-1)
	  read(varchar,*,iostat=ios) nmobiles
   ELSE IF (INDEX(varname,'nbmicro')>0) THEN         ! Specify begining of indoor micros
      i = INDEX(varchar,'!')                         ! Look for comment
      IF (i>0) varchar=varchar(1:i-1)                ! Remove comment
	  read(varchar,*,iostat=ios) nbmicro			 !
   ELSE IF (INDEX(varname,'nemicro')>0) THEN         ! Specify ending of indoor micros
      i = INDEX(varchar,'!')                         ! Look for comment
      IF (i>0) varchar=varchar(1:i-1)                ! Remove comment
	  read(varchar,*,iostat=ios) nemicro			 !	
   ELSE IF (INDEX(varname,'nvehicles')>0) THEN       ! Specify micros which are commuting means
      i = INDEX(varchar,'!')                         ! Look for comment
      IF (i>0) varchar=varchar(1:i-1)                ! Remove comment
	  read(varchar,*,iostat=ios) nvehicles			 !
   ELSE IF (INDEX(varname,'npublict')>0) THEN        ! Specify micros which are public transit
      i = INDEX(varchar,'!')                         ! Look for comment
      IF (i>0) varchar=varchar(1:i-1)                ! Remove comment
	  read(varchar,*,iostat=ios) npublict			 !	  		      
   ELSE IF (INDEX(varname,'comm')>0) THEN            ! Set commuting flag
      cf=varchar(1:1)                                ! Read first character
      IF ((cf=='Y').OR.(cf=='y')) commut=2           ! If YES then set to ON
   ELSE                                              ! Or else set parameters
      if (varname =='backg') then
        READ(varchar,*,IOSTAT=ios) avarint           ! Convert char to integer
        bconc = avarint
      end if
      if (varname =='sarod') then
        READ(varchar,*,IOSTAT=ios) sarod             ! sarod code
      end if
      READ(varchar,*,IOSTAT=ios) varint              ! Convert char to integer
      IF (ios/=0) CYCLE                              ! If not integer, discard
      IF (varname=='nmicro')  nmicro =varint         ! Set # microenvironments
      IF (varname=='hblock')  hblock =varint         ! Set # time block per day
      IF (varname=='ntype')   ntype  =varint         ! Set # day types
      IF (varname=='ngroup')  ngroup =varint         ! Set # demographic groups
      IF (varname=='nsource') nsource=varint         ! Set # seasons per year
      IF (varname=='year')    year   =varint         ! Set model year
      IF (varname=='region1') region1=varint
      IF (varname=='region2') region2=varint
	  IF (varname=='Rseed1')  idum=varint			 ! Set random seed for activity pattern
	  IF (varname=='Rseed2')  idum2=varint           ! Set random seed for micro factors
	  IF (varname=='Rseed3')  idum3=varint		     ! Set random seed for air quality data
	  IF (varname=='nreplic') then 
		  ISP1P2=1
		  amax=varint
	  END IF
   END IF                                              ! Fininshed with line
END DO                                                 ! Get next PARAMS record
!

commut=2											   ! to block non-commuting option 9/23/2005

idum21=idum2											! get a copy of random seed

CLOSE(10)                                              ! Close PARAMS file
!
IF (ISP1P2==0) then
	write (*,*) 'Wrong Parameter File.  Use P2.txt for this module'
	STOP
END IF 
!  Setup section
nmsource=0
do i=1, 10
	if(nmobiles(i)/=0) then
	nmsource=nmsource+1
!	nmobiles2(nmsource)=nmobiles(i)
	end if
end do 

do i=1, nmsource

if(nmobiles(i) > nsource) then
write(*,*) 'ERROR: nmobiles in parameter file must be a number less than nsource'
stop
end if
end do 


nmvehicle=0
do i=1,10
	if(nvehicles(i)/=0) then
	nmvehicle=nmvehicle+1
	end if

end do

nmpublict=0

do i=1,10
	if(npublict(i)/=0) then
	nmpublict=nmpublict+1
	end if

end do



!write(*,*) nmpublict,npublict
!stop

!
i=INDEX(f13,'.')-1                                    ! Locate extension on f13
IF (i==0) i=LEN_TRIM(f13)                             ! Trim f13 file name
f92=f13(:i)//'.state_air1_fip_range'
f93=f13(:i)//'.state_air_fip_range'
f35=f13(:i)//'.pop_air_da'
f23=f13(:i)//'.da'								! Add file extension
f43=f13(:i)//'.air_da'                          ! Add direct access extension
f94=f13(:i)//'.state_air2_fip_range'            ! Add direct access extension
i=INDEX(f11,'.')-1                              ! Locate extension on file 11
IF (i==0) i=LEN_TRIM(f11)                       ! Trim off extension
f21=f11(:i)//'.da'                              ! Add direct access extension
f30=f11(:i)//'.nonzero'
!
i=INDEX(f15,'.')-1
IF(i==0) i=LEN_TRIM(f15)
f151=f15(:i)//'.add'
f152=f15(:i)//'.pen'
f153=f15(:i)//'.prox'
!
IF (commut == 2) then                           ! if there is commuting, set files names
  i=INDEX(f19,'.')-1                              ! Locate extension on f19
  IF (i==0) i=LEN_TRIM(f19)                       ! Trim f19 file name
  f29=f19(:i)//'.da'                              ! Index file name
  f39=f19(:i)//'.ind'                             ! Index file name
  f94=f19(:i)//'st_comm1_fip_range'
END IF

ip1=LEN_TRIM(CAS)
ip2=LEN_TRIM(EPAregion)
ip3=LEN_TRIM(f1511)
f1512 = f1511(:ip3)//CAS(:ip1)//'\'//'product\'//CAS(:ip1)//'P_R'//EPAregion(:ip2)//'.txt'
f1513 = f1511(:ip3)//CAS(:ip1)//'\'//'material_old\'//CAS(:ip1)//'M1_R'//EPAregion(:ip2)//'.txt'
f1514 = f1511(:ip3)//CAS(:ip1)//'\'//'material_new\'//CAS(:ip1)//'M2_R'//EPAregion(:ip2)//'.txt'
f1515 = f1511(:ip3)//CAS(:ip1)//'\'//'product\'//CAS(:ip1)//'P_Fra'//'.txt'
f1516 = f1511(:ip3)//CAS(:ip1)//'\'//'material_old\'//CAS(:ip1)//'M1_fra'//'.txt'
f1517 = f1511(:ip3)//CAS(:ip1)//'\'//'material_new\'//CAS(:ip1)//'M2_fra'//'.txt'

f1518 = f1511(:ip3)//CAS(:ip1)//'\'//'combustion\'//CAS(:ip1)//'C_fra'//'.txt'
f1519 = f1511(:ip3)//CAS(:ip1)//'\'//'combustion\'//CAS(:ip1)//'C_R'//EPAregion(:ip2)//'.txt'
f1520 = f1511(:ip3)//CAS(:ip1)//'\'//'Vehicle\'//CAS(:ip1)//'V.txt'


OPEN (15,FILE=f15, ACTION = 'READ')    ! Open distribution factors file
i=LEN_TRIM(f15111)
f15111=f15111(1:i)
OPEN (15111,FILE=f15111,ACTION='READ') ! Open mobile factors file
!write(*,*) f1611
OPEN (1611,FILE=f1611,ACTION='READ')   ! Open auto product id file
OPEN (18,FILE=f18,POSITION='APPEND',ACTION='WRITE')      ! Open log file
!OPEN (21,FILE=f21,ACCESS='DIRECT',RECL=4*nmicro*hblock*commut)
OPEN (21,FILE=f21,STATUS='old',ACTION='READ')
OPEN (30,FILE=f30,status='old',ACTION='READ')
OPEN (31,FILE=f31,ACTION='READ')
!OPEN (35,FILE=f35,ACCESS='DIRECT',RECL=4*ngroup)
OPEN (35,FILE=f35,ACTION='READ')
!OPEN (43,FILE=f43,ACCESS='DIRECT',RECL=8*hblock*nsource*250)
OPEN (43, FILE=f43,ACTION='READ')
OPEN (23,FILE=f23,ACTION='READ')
OPEN (60,FILE=f60,position='APPEND',action='write')
OPEN (92,FILE=f92,ACTION='READ')
OPEN (93,FILE=f93,ACTION='READ')
OPEN (94,FILE=f94,ACTION='READ')
OPEN (99,FILE=f99,status='old')
OPEN (151, FILE=f151, ACTION='write')
OPEN (152, FILE=f152, ACTION='write')
OPEN (153, FILE=f153, ACTION='write')
!OPEN (1111,FILE='test.txt',ACTION='write')
OPEN (1512,FILE=f1512,ACTION='read',IOSTAT=IERROR1)
!OPEN (9000, FILE='temp6.txt',action='write')
!OPEN (9001, FILE='temp7.txt',action='write')
!OPEN (9002, FILE='temp8.txt',action='write')
if(IERROR1==0) then 
write(*,*) 'Indoor emission from consumer products              = Included'
else
write(*,*) 'Indoor emission from consumer products              = Not included'
end if

OPEN (1513,FILE=f1513,ACTION='read',IOSTAT=IERROR2)
if(IERROR2==0) then
write(*,*) 'Indoor emission from construction materials         = Included'
else
write(*,*) 'Indoor emission from construction materials         = Not included'
end if

OPEN (1514,FILE=f1514,ACTION='read',IOSTAT=IERROR3)
if(IERROR3==0) then 
write(*,*) 'Indoor emission from freshly installed materials    = Included'
else
write(*,*) 'Indoor emission from freshly installed materials    = Not included'
end if

OPEN (1519,FILE=f1519,ACTION='read',IOSTAT=IERROR8)
if(IERROR8==0) then 
write(*,*) 'Indoor emission from combustion sources             = Included'
else
write(*,*) 'Indoor emission from combustion sources             = Not included'
end if

OPEN(1520,FILE=f1520,ACTION='read', IOSTAT=IERROR9)
if(IERROR9==0) then
write(*,*) 'Indoor emission from vehicles in residential garage = Included'
else
write(*,*) 'Indoor emission from vehicles in residential garage = Not included'
end if

OPEN (1515,FILE=f1515,ACTION='read',IOSTAT=IERROR4)
if(IERROR4/=0 .AND. IERROR1==0) then
write(*,*) 'Error: No file for product building fractions is found'
stop
end if

OPEN (1516,FILE=f1516,ACTION='read',IOSTAT=IERROR5)
if(IERROR5/=0 .AND. IERROR2==0) then
write(*,*) 'Error: No file for material building fractions is found'
stop
end if

OPEN (1517,FILE=f1517,ACTION='read',IOSTAT=IERROR6)
if(IERROR6/=0 .AND. IERROR3==0) then 
write(*,*) 'Error: No file for new material building fractions is found'
stop
end if

OPEN (1518,FILE=f1518,ACTION='read',IOSTAT=IERROR7)
if(IERROR7/=0 .AND. IERROR8==0) then 
write(*,*) 'Error: No file for combustion building fractions is found'
stop
end if





if (IERROR1==0 .AND. IERROR4==0) YPRODUCT=0
if (IERROR2==0 .AND. IERROR5==0) YM1=0
if (IERROR3==0 .AND. IERROR6==0) YM2=0
if (IERROR7==0 .AND. IERROR8==0) YCOMB=0
if (IERROR9==0) then
	YAUTO=0
	Indauto=1
end if

!write(*,*) YPRODUCT,YM1,Ym2

if (commut == 2) then                            ! if there is commuting, open relevant files
  OPEN (29,FILE=f29,ACTION='READ')
  OPEN (39,FILE=f39,ACTION='READ')
endif
!
!
1509 format (A11,9F11.3)
1609 format (A11,12F11.3)
1510 format (3I5,9E11.3)


ALLOCATE (productid(MaxProd),material1(MaxProd), material2(MaxProd),combid(MaxProd))
ALLOCATE (prodfra(MaxProd,9),m1fra(MaxProd,9),m2fra(MaxProd,9),combfra(MaxProd,9))
ALLOCATE (prodconc(MaxProd,9,8,10),m1conc(MaxProd,9,8,10),m2conc(MaxProd,9,8,10),combconc(MaxProd,9,8,10),autoconc(MaxProd,2,8,10))
ALLOCATE (prodfra2(MaxProd,nmicro),m1fra2(MaxProd,nmicro),m2fra2(MaxProd,nmicro),combfra2(MaxProd,nmicro))
ALLOCATE (prodconc2(MaxProd,nmicro,8,10),m1conc2(MaxProd,nmicro,8,10),m2conc2(MaxProd,nmicro,8,10),combconc2(MaxProd,nmicro,8,10),autoconc2(MaxProd,nmicro,8,10))
ALLOCATE (ambh(hblock,nsource,MaxNo))
ALLOCATE (amback(MaxNo),ambackw(MaxNo))
ALLOCATE (amblh(hblock,nsource),ambh2(hblock,nsource))                   ! Dynamic allocation
ALLOCATE (ambw(hblock,nsource,MaxNo))
ALLOCATE (amblw(hblock,nsource), ambw2(hblock,nsource))                   ! Dynamic allocation
ALLOCATE (prox(nmicro,nsource),addP(nmicro,hblock),addM(nmicro,hblock), addC(nmicro,hblock),addT(nmicro,hblock))      ! Dynamic allocation
ALLOCATE (pen(nmicro,hblock),lag(nmicro,nsource,maxdist))       ! Dynamic allocation
ALLOCATE (durw(nmicro*hblock,commut),dur(nmicro*hblock,commut,ncom,ntype,ngroup,maxcls,max1),dursc(nmicro*hblock,commut,ntype))   ! Dynamic allocation
ALLOCATE (durs(nmicro*hblock, commut), dursc2(nmicro*hblock,commut))
ALLOCATE (conc(nmicro*hblock,commut,nsource))            ! Dynamic allocation
ALLOCATE (concb(nmicro*hblock,commut),concaddP(nmicro*hblock,commut),concaddM(nmicro*hblock,commut),concaddC(nmicro*hblock,commut),concaddT(nmicro*hblock,commut))   ! Dynamic allocation
ALLOCATE (ival(ngroup,ntype,ncom,maxcls),jcom(ngroup,ntype),clusno(ngroup,ntype,ncom),fact(ntype))
ALLOCATE (bmax(ntype),rand(ntype),randcls(ntype,maxcls))
ALLOCATE (penA(maxFNO, nmicro), addA(maxFNO,nmicro),proxA(maxFNO,nsource, nmicro,maxdist))
ALLOCATE (TempArray(6,3,nsource,nmicro), RandomOut(nmicro,nsource,3), DataArray(10))
ALLOCATE (Freq(ngroup,ntype,ncom,maxcls),Prob(ngroup,ntype,ncom,maxcls,maxcls),Prob1(ngroup,ntype,ncom,maxcls*maxcls))
ALLOCATE (Clsdays(ntype,maxcls),clsdays2(maxcls),clsdays1(maxcls),Prob11(maxcls,maxcls),Freq1(maxcls))
ALLOCATE (autoprod(maxauto),autopen(maxauto,maxFNO))
ALLOCATE (RD2(nmsource))
ALLOCATE (zoneidxh(nmicro,nsource),zoneidxw(nmicro,nsource))
!
write (*,*) " durs length = ", nmicro*hblock*commut
write (60,*) 'MISSING TRACTS of AIRQUAL & COMMUTE IN HAPEM'

! Read the distribution factors for micro-environmental factors

write(*,*) 'Loading data ...'

productid=''
prodfra=0
prodconc=0

material1=''
m1fra=0
m1conc=0

material2=''
m2fra=0
m2conc=0

combid=''
combfra=0
combconc=0

autoconc=0

autoprod=''
autopen=0

nat=0

IF(YPRODUCT==0) then				! Read auto garage file
	read(1611,*)
	Do 
		read(1611,*,IOSTAT=ios) 
		if(ios/=0) exit
		nat=nat+1
	End do

	Rewind(1611)
	read(1611,*)
	Do i=1, nat

		read(1611,1609) autoprod(i),(autopen(i,g),g=1,10)

	end do 
END IF

npt=0
If (YPRODUCT==0) then			 ! read files for indoor consumer products
	Do 
	read(1515,*,IOSTAT=ios)
	if(ios/=0) exit
	npt=npt+1
	End do 
	
	rewind(1515)
	
	Do i=1, npt
		read(1515,1509) productid(i),(prodfra(i,g),g=1,9)
!		write(*,*) productid(i),(prodfra(i,g),g=1,9)
		Do j1=1,9
			Do k1=1,10
				read(1512,1510) id1,id2,id3,(prodconc(i,j1,j,k1),j=1,8)
			end do 
		End do 	
	End do 
end if
!stop
nmt1=0
If (YM1==0) then			 ! if there are files for indoor consumer materials, read them
	Do 
	read(1516,*,IOSTAT=ios)
	if(ios/=0) exit
	nmt1=nmt1+1
	End do 

	rewind(1516)

	Do i=1, nmt1
		read(1516,1509) material1(i),(m1fra(i,g),g=1,9)
		Do j1=1,9
			Do k1=1,10
				read(1513,1510) id1,id2,id3,(m1conc(i,j1,j,k1),j=1,8)
			end do 
		End do	
	End do 
end if

nmt2=0
If (YM2==0) then			 ! if there are files for indoor consumer materials, read them
	Do 
	read(1517,*,IOSTAT=ios)
	if(ios/=0) exit
	nmt2=nmt2+1
	End do 

	rewind(1517)

	Do i=1, nmt2
		read(1517,1509) material2(i),(m2fra(i,g),g=1,9)
		Do j1=1, 9
			Do k1=1,10
				read(1514,1510) id1,id2,id3,(m2conc(i,j1,j,k1),j=1,8)
			end do 
		End do 	
	End do 
end if

nct=0
If (YCOMB==0) then			 ! read files for combustion sources
	Do 
	read(1518,*,IOSTAT=ios)
	if(ios/=0) exit
	nct=nct+1
	End do 
	
	rewind(1518)
	
	Do i=1, nct
		read(1518,1509) combid(i),(combfra(i,g),g=1,9)
!		write(*,*) productid(i),(prodfra(i,g),g=1,9)
		Do j1=1,9
			Do k1=1,10
				read(1519,1510) id1,id2,id3,(combconc(i,j1,j,k1),j=1,8)
			end do 
		End do 	
	End do 
end if

if(YAUTO==0) then

	DO i=1,1
		Do j1=1,2
			Do k1=1,10
				read(1520,1510) id1,id2,id3,(autoconc(i,j1,j,k1),j=1,8)
			end do
		End do 
	END DO

END IF 


!write(*,*) npt,nmt1,nmt2

prodfra2=0
prodconc2=0
m1fra2=0
m1conc2=0
m2fra2=0
m2conc2=0
combfra2=0
combconc2=0

autoconc2=0

if (YPRODUCT==0 .OR. YM1==0 .OR. YM2==0 .OR. YCOMB==0) then

DO i=1, nmicro                   ! mapping SRD micro # to HAPEM micro #

	if(i==1 .OR. i==2) then  ! mapping SRD residence (non products) to hapem residences
!		prodfra2(:,i) = prodfra(:,1)
!		prodconc2(:,i,:,:) = prodconc(:,1,:,:)
		combfra2(:,i) = combfra(:,1)
		m1fra2(:,i) = m1fra(:,1)
		m2fra2(:,i) = m2fra(:,1)
		combconc2(:,i,:,:) = combconc(:,1,:,:)
		m1conc2(:,i,:,:) = m1conc(:,1,:,:)
		m2conc2(:,i,:,:) = m2conc(:,1,:,:)
	end if
!write(*,*) i
	if(i==1 .OR. i==2) then  ! mapping SRD/hapem product residence mapping
		
		prodfra2(:,i) = prodfra(:,1)
		
		DO j=1,npt

			Do ki=1,nat									! check if a product is an auto-related product
				if (productid(j) == autoprod(ki)) then
					autoY=0
					autofind=ki
!					write(*,*) ki
					exit
				end if
			End do 

			if(autoY==0) then							! mapping conc. according to if a product is an auto product

				if(i==11) then
					prodconc2(j,i,:,:) = prodconc(j,1,:,:)
				end if

				if(i==13 .OR. i==14) then
					prodconc2(j,i,:,:) = prodconc(j,1,:,:)*0.0
				end if

				if(i==15 .OR. i==16) then
					call Random2(ProSel,idum2,10)
					prodconc2(j,i,:,:) = prodconc(j,1,:,:)*autopen(autofind,ProSel)
!					write(*,*) i,j,autofind,ProSel,autopen(autofind,ProSel)
				end if

			else if(autoY==1) then

!				if(i==11) then
!					call Random2(ProSel,idum2,10)
!					prodconc2(j,i,:,:) = prodconc(j,1,:,:)*autopen(autofind,ProSel)
!				end if
!				if(i==13 .OR. i==14 .OR. i==15 .OR. i==16) then
					prodconc2(j,i,:,:) = prodconc(j,1,:,:)
!				end if
			else

				write(*,*) 'code error in garage penetration factor for auto products'
				stop
			end if										! end of conccentration mapping
			autoY=1
			autofind=1
		
		End do											! end of checking if a product is autoproduct
	
	End if												! end of mapping Product-specific SRD/hapem residence mapping 
	

	if(i==3) then							! mapping SRD school to hapem school
		prodfra2(:,i) = prodfra(:,2)
		combfra2(:,i) = combfra(:,2)
		m1fra2(:,i) = m1fra(:,2)
		m2fra2(:,i) = m2fra(:,2)
		prodconc2(:,i,:,:) = prodconc(:,2,:,:)
		combconc2(:,i,:,:) = combconc(:,2,:,:)
		m1conc2(:,i,:,:) = m1conc(:,2,:,:)
		m2conc2(:,i,:,:) = m2conc(:,2,:,:)
	end if

	if(i==4) then							! mapping SRD hospital to hapem hospital
		prodfra2(:,i) = prodfra(:,3)
		combfra2(:,i) = combfra(:,3)
		m1fra2(:,i) = m1fra(:,3)
		m2fra2(:,i) = m2fra(:,3)
		prodconc2(:,i,:,:) = prodconc(:,3,:,:)
		combconc2(:,i,:,:) = combconc(:,3,:,:)
		m1conc2(:,i,:,:) = m1conc(:,3,:,:)
		m2conc2(:,i,:,:) = m2conc(:,3,:,:)
	end if 

	if(i==5) then						    ! mapping SRD office to hapem office
		prodfra2(:,i) = prodfra(:,5)
		combfra2(:,i) = combfra(:,5)
		m1fra2(:,i) = m1fra(:,5)
		m2fra2(:,i) = m2fra(:,5)
		prodconc2(:,i,:,:) = prodconc(:,5,:,:)
		combconc2(:,i,:,:) = combconc(:,5,:,:)
		m1conc2(:,i,:,:) = m1conc(:,5,:,:)
		m2conc2(:,i,:,:) = m2conc(:,5,:,:)
	end if 
											! mapping SRD public access building

	if(i==6) then
		prodfra2(:,i) = prodfra(:,6)
		combfra2(:,i) = combfra(:,6)
		m1fra2(:,i) = m1fra(:,6)
		m2fra2(:,i) = m2fra(:,6)
		prodconc2(:,i,:,:) = prodconc(:,6,:,:)
		combconc2(:,i,:,:) = combconc(:,6,:,:)
		m1conc2(:,i,:,:) = m1conc(:,6,:,:)
		m2conc2(:,i,:,:) = m2conc(:,6,:,:)
	end if	
	
	
	if(i==7) then						  ! mapping SRD restraunt to hapem restraunt
		prodfra2(:,i) = prodfra(:,8)
		combfra2(:,i) = combfra(:,8)
		m1fra2(:,i) = m1fra(:,8)
		m2fra2(:,i) = m2fra(:,8)
		prodconc2(:,i,:,:) = prodconc(:,8,:,:)
		combconc2(:,i,:,:) = combconc(:,8,:,:)
		m1conc2(:,i,:,:) = m1conc(:,8,:,:)
		m2conc2(:,i,:,:) = m2conc(:,8,:,:)
	end if 

	if(i==8 .or. i==9) then  ! mapping SRD car to hapem vehicle micros

		prodfra2(:,i) = prodfra(:,9)
		combfra2(:,i) = combfra(:,9)
		m1fra2(:,i) = m1fra(:,9)
		m2fra2(:,i) = m2fra(:,9)
		prodconc2(:,i,:,:) = prodconc(:,9,:,:)
		combconc2(:,i,:,:) = combconc(:,9,:,:)
		m1conc2(:,i,:,:) = m1conc(:,9,:,:)
		m2conc2(:,i,:,:) = m2conc(:,9,:,:)
	end if


end do
End if										! End of mapping SRD micros to HAPEM micros
!STOP
if(YAUTO==0) then							! Mapping vehicle in garage

	DO i=1, nmicro
			if(i==2) then 
			autoconc2(:,i,:,:)=autoconc(:,1,:,:)
			end if
			if(i==1) then
			autoconc2(:,i,:,:)=autoconc(:,2,:,:)
			end if
	End do 
end if 


deallocate(prodfra, m1fra,m2fra,prodconc,m1conc,m2conc,combfra,combconc, autoconc)



DO s1 = 1, nmicro

	READ(15,*) micron, penA(:,s1),addA(:,s1),(proxA(:,s,s1,1), lag(s1,s,1), s=1, nsource)
 
END do
! if there is mobile sources, overwrite the proxifactors with values in mobile factor files
if(nmsource>0) then ! begin of mobile source factor input

	nmrecord=0	
	Do 
		read(15111,*,IOSTAT=ios)
		if(ios/=0) exit
		nmrecord=nmrecord+1
	end do
	rewind(15111) 
	nmsource2=nmrecord/nmicro
	if(nmsource2 /=nmsource) then
		write(*,*) 'ERROR: # of mobile sources not matched in para and mobile factor files'
		write(*,*) nmsource2
		write(*,*) nmsource
		stop
	else

		Do i=1, nmsource
			Do s1=1,nmicro
			read(15111,*,IOSTAT=ios) micron,(proxA(:,nmobiles(i),s1,s),lag(s1,nmobiles(i),s),s=1,maxdist)	
			end do 
		end do 
    end if 


! find average proxifactor in D2 region

RD2=0.0
nRD2=0

Do i = 1, nmsource
!nmobiles(i)
	Do k=nbmicro, nemicro
		
				if(proxA(1,nmobiles(i),k,2) == 1 .or. proxA(1,nmobiles(i),k,2) == 2) then
					RD2(i) = RD2(i) + proxA(2,nmobiles(i),k,2)
				else if (proxA(1,nmobiles(i),k,2) == 3) then
					RD2(i) = RD2(i) + (proxA(2,nmobiles(i),k,2)+proxA(3,nmobiles(i),k,2))/2.0
				else if (proxA(1,nmobiles(i),k,2) == 4) then
					RD2(i) = RD2(i) + proxA(4,nmobiles(i),k,2)
				else if (proxA(1,nmobiles(i),k,2) == 5) then
					RD2Q=0.0
					Do j=1,proxA(2,nmobiles(i),k,2)
						RD2Q=RD2Q+proxA((2+j),nmobiles(i),k,2)
					END DO
					RD2(i) = RD2(i) + RD2Q/proxA(2,nmobiles(i),k,2)
				else
					write(*,*) ' Error with mobile source factor files'
					stop
				end if
	end do 

	RD2(i) = RD2(i)/(nemicro-nbmicro+1)

End do
 
End if		! end of mobile source factor input
!write(*,*) (RD2(i), ' ', i=1, nmsource)
!stop
		

CLOSE (15)  ! Close factors file
                  
101 format (10(i5))
!
bmax(1) = 66.0
bmax(2) = 195.0
bmax(3) = 104.0
!
dur = 0
!
do g = 1, ngroup
	do t=1,ntype
		do dcom=1,ncom
			read (30,101) igroup,iday,icomm,(ival(g,t,dcom,cls), cls = 1, maxcls),clusno(g,t,dcom),jcom(g,t)
		end do 
    end do 
end do
!
read(31,*)

do g = 1, ngroup
	do t=1,ntype
		do dcom=1,ncom
		read(31,*) igroup,iday,icomm,icluster,(Freq(g,t,dcom,i),i=1,maxcls),(Prob1(g,t,dcom,j),j=1,maxcls*maxcls)
        prob(g,t,dcom,1,:)=prob1(g,t,dcom,1:3)
		prob(g,t,dcom,2,:)=prob1(g,t,dcom,4:6)
		prob(g,t,dcom,3,:)=prob1(g,t,dcom,7:9)
		end do 
	end do 
end do


!DO g=1, ngroup                                      ! For each dem. group
!	DO t=1,ntype                                        ! For each day type
!		DO dcom=1,ncom										! For each com
!		Do cls=1,clusno(g,t,dcom)								! For each cluster
!			write(*,*) 
!			 if (ival(g,t,dcom,cls) /= 0) then
!				DO isam = 1, ival(g,t,dcom,cls)
!					isum = isum + 1
!					write(67,*) isum,g,t,dcom,cls
!				END DO
!			 end if
!			END DO 
!		END DO
!	END DO
!END DO

!isum=0
!DO g=1, ngroup                                      ! For each dem. group
!	DO t=1,ntype                                        ! For each day type
!		DO dcom=1,ncom										! For each com
!		Do cls=1,clusno(g,t,dcom)								! For each cluster
!			write(*,*) 
!			 if (ival(g,t,dcom,cls) /= 0) then
!				DO isam = 1, ival(g,t,dcom,cls)
!					isum = isum + 1
!						READ (21,211,IOSTAT=ios) isum2,g2,t2,dcom2,cls2,isam2,durs(:,:)
!						write(*,*) " durs ", isum2, g2, t2, dcom2, cls2, isam2
!						if (isum2==isum .AND. g2==g .AND. t2==t .AND. dcom2==dcom .AND. cls2==cls .AND. isam2==isam) then 
!						write(9000,*) dcom,sum(durs),sum(durs(:,2))
!						dur(:,:,dcom,t,g,cls,isam) = real(durs(:,:))
!						write(67,*) isum,g,t,dcom,cls
!						else
!						write(*,*) 'problems in reading file 21'
!						write(*,*) isum2,isum,g2,g,t2,t,dcom2,dcom,cls2,cls,isam2,isam
!						stop
!						end if 
!				END DO
!			 end if
!			END DO 
!		END DO
!	END DO
!END DO

211 format (I7,I3,I3,I3,I3,I7,288I5)
DO
  READ (21,211,IOSTAT=ios) isum, g, t, dcom, cls, isam, durs(:,:) 
  IF(ios) EXIT 
  dur(:,:,dcom,t,g,cls,isam) = real(durs(:,:))
ENDDO
    
!Write(*,*) 'finish normal'

!Close (67)
!   
n=nmicro*hblock

do g = 1, ngroup
  do t = 1, ntype
   do dcom=1,ncom ! com
   do cls=1,clusno(g,t,dcom)
    if (ival(g,t,dcom,cls) /= 0) then
      do isam = 1, ival(g,t,dcom,cls)
        total = 0.0
        do ii = 1, n
          do jj = 1, commut
            total = total+dur(ii,jj,dcom,t,g,cls,isam)
          end do
        end do
        if (abs(total-1440.0) > 0.002) then 
          write (18,*) 'total time per activity day > 1440',g,t,isam, total
		  write (*,*) 'total time per activity day > 1440',g,t,isam, total
          stop
        end if
      end do
	 end if
	end do
	end do ! com
  end do
end do
!
CLOSE(21)                                                 ! Close activity file

!write(*,*) 'step3'

CALL SYSTEM_CLOCK(cl2,cr,cm)                              ! Get job time
!
!  Exposure section
!
read (99,*) itest, ktest
read (99,*) itract, acount
IF (commut == 2) then
  read (99,*) ntrac14, ntrac19
  read (99,*) wtract, cnum
  ALLOCATE (frworkers(wtract), wtractid(wtract), DISTW(maxring,ngroup,wtract),AREAW(maxring,wtract),distfra(wtract))
  ALLOCATE (ctractid(cnum), first(cnum), last(cnum), iflag(cnum),DISTH(maxring,ngroup,cnum), AREAH(maxring,cnum))
  ALLOCATE (compro(2,cnum),comtime(2,cnum),fracpop(fracmax,ngroup,cnum))
ENDIF
read (99,*) atract, ifip
read (99,*) atract2,airecord
!
80 format (a,2x,i6,2x,i6,2x,i6)
81 format (i2, 6x, a, 6x, i2)
231 format (I14,200E14.6)
!
ALLOCATE (atractid(atract), DISTHA(maxring,ngroup,atract),AREAHA(maxring,atract),first1(atract),last1(atract),n1(atract), aqout(hblock,nsource,airecord),backC(airecord))
ALLOCATE (ftractid(ifip), psump(ifip), asuma(ifip))
ALLOCATE (popf(ngroup))
ALLOCATE (tconc(ngroup,amax))
ALLOCATE (cexp(ngroup,nsource,amax),fexp(ngroup,amax),fexpaP(ngroup,amax),fexpaM(ngroup,amax),fexpaC(ngroup,amax),fexpaT(ngroup,amax))     ! Dynamic allocation
ALLOCATE (pop(ngroup,atract))
!
!write(*,*) 'step1'

do ii = 1, maxst				  ! Read state-to-tract indices
	read (94,*,IOSTAT=ios) astind(ii),stp(ii),sta(ii)     ! for the national population file
end do

do ii = 1, ifip			                ! Read county-to-tract indices
  read (93,*,IOSTAT=ios) ftractid(ii), psump(ii), asuma(ii)  ! for air quality file
end do

do ii = 1, maxst			        ! Read state-to-county indices
  read (92,*,IOSTAT=ios) bstind(ii), stsump(ii), stsuma(ii)	 ! for air quality file
end do

do ii = 1, atract
  read (35,*,IOSTAT=ios) pop(:,ii)	          ! Read the population and air quality
  read (43,*,IOSTAT=ios) atractid(ii), first1(ii),last1(ii),AREAHA(:,ii),(DISTHA(:,s,ii),s=1,ngroup) ! data for each tract in the air 
end do						      ! quality file

do ii=1, airecord
  read(23,231,IOSTAT=ios) arecord, aqout(:,:,ii),backC(ii) 
end do 
!write(*,*) 'step2'
if (commut == 2) then
   do ii = 1, cnum					      ! Read home tract-to-work
      read (39,*,IOSTAT=ios) ctractid(ii), first(ii), last(ii), iflag(ii),compro(:,ii), comtime(:,ii),AREAH(:,ii),(DISTH(:,s,ii),s=1,ngroup), (fracpop(:,ik,ii),ik=1,ngroup) ! tract indices for
   end do								! commuting file
   do ii = 1, wtract			     ! Read home tract worker fractions
      read (29,*,IOSTAT=ios) wtractid(ii),frworkers(ii),AREAW(:,ii),(DISTW(:,s,ii),s=1,ngroup), distfra(ii)		 ! for each work tract
   end do
end if 

!write(*,*) 'step3'
!write (*,*) 'Processing data ....'

1001 format ('Processing',I6,'  of',I6)
do ist = region1, region2
!
!   write(*,*) sarod
!   write(*,*) astind(ist)
!   write(*,*) path1
    f61 = TRIM(path1)//sarod//'.'//astind(ist)//'.'//'dat'
!   f611=gz//TRIM(path1)//sarod//'.'//astind(ist)//'.'//'d'//'.'//'dat'
  
!   write(*,*) f61  
    OPEN (61,FILE=f61,RECL=11+ngroup*(12*nsource+20),ACTION='WRITE')
!
  READ(EPAregion,*,iostat=ios) EPAreg

  WRITE(61,*)  '****** HAPEM OUTPUT FILE ******'
  WRITE(61,*) 
  WRITE(61,*)  'State fips                        = ',astind(ist)
  WRITE(61,*)  'SAROD                             = ',SAROD
  WRITE(61,*)  'Pollutant                         = ',pollutant
  WRITE(61,*)  'CAS #                             = ',CAS
  WRITE(61,*)  'Air Quality Data Units            = ',units
  WRITE(61,*)  'Year of Air Quality Data          = ',year
  WRITE(61,*)  'Outdoor Air Source Categories     = ',nsource
  WRITE(61,*)
  Write(61,*)  'Random Seed for Activity Pattern  = ',idum
  WRITE(61,*)  'Random Seed for Micro Factors     = ',idum21
  WRITE(61,*)  'Random Seed for Air Quality Data  = ',idum3 
  WRITE(61,*)
  WRITE(61,*)  'Total indoor product sources      = ',npt
  WRITE(61,*)  'Total indoor material sources     = ',nmt1
  WRITE(61,*)  'Total indoor combustion sources   = ',nct
  WRITE(61,*)  'Vehicle emission in resid. garage = ',indauto
  WRITE(61,*)  'EPA Region of Indoor Add Files    = ',EPAreg
  WRITE(61,*)
  WRITE(61,*)  'Total Demographic Groups          = ',ngroup
  WRITE(61,*)  'Total Replicates for each group   = ',amax
  WRITE(61,*)  'Group 1  B_00  = ',M1
  WRITE(61,*)  'Group 2  B_02  = ',M2
  WRITE(61,*)  'Group 3  B_05  = ',M3
  WRITE(61,*)  'Group 4  B_16  = ',M4
  WRITE(61,*)  'Group 5  B_18  = ',M5
  WRITE(61,*)  'Group 6  B_65  = ',DF1
  WRITE(61,*)
611 format (A3,A4,A7,A5,A6,30A11)
  write(61,*)
  WRITE(61,611)  'ST',' CTY',' CENSUS ','GRUP',' POPUL',(SOURCEID(idx20),idx20=1,nsource),'  BackgConc',' IndCon_Pro',' IndCon_Mat',' IndCon_Com',' IndCon_Veh',' Total Conc'
  do jc = stp(ist), sta(ist)
    ifnd  = 0
    ifnd1 = 0
    ambh = 0.0
	amback= 0.0
	amback2=0.0
	ambh2 = 0.0
    amblh = 0.0
	if(commut==1) then
	write(*,1001) jc, atract
	end if 
    IF (commut == 2) then                                !If commuting is included,
	  write(*,1001) jc, cnum
      do ii = stsump(ist),stsuma(ist)			 !step through the commuting tracts,
        if (ftractid(ii) == ctractid(jc)(1:5)) then      !matching them with air quality 
          ifnd = ii                                      !tracts 
          exit
        end if
      end do
      if (ifnd == 0) then
        write (60,104) 'home', ist, jc, ctractid(jc), iflag(jc)
        cycle
      end if     
      do ii = psump(ifnd), asuma(ifnd)
        if (atractid(ii) == ctractid(jc)) then
          n11=0
		  do i3=first1(ii),last1(ii)
		    n11 = n11+1
			ambh(:,:,n11) = aqout(:,:,i3)
			amback(n11) = backC(i3)
		  end do 
		  AirNo = n11
          popf(:) = real(pop(:,ii))/real(amax)
          ifnd1 = ii
          exit
        end if
      end do
      if (ifnd1 == 0) then
        write (60,104) 'home',ist, jc, ctractid(jc), iflag(jc)
        cycle
      end if
      jtest = jtest + 1
!	  write(*,*) jtest
    ELSE                                                 !Otherwise, just step through the
	  n11 = 0
      do i3=first1(jc),last1(jc)
	  n11=n11+1
	  ambh(:,:,n11) = aqout(:,:,i3)                          !air quality tracts
	  amback(n11) = backC(i3)
 	  end do 
	  AirNo = n11
      popf(:) = real(pop(:,jc))/real(amax)
      ntrac14 = ntrac14 + 1               
    ENDIF
!
!
!
  
    cexp = 0.0
    fexp = 0.0
	fexpaP = 0.0
	fexpaM = 0.0
	fexpaC = 0.0
	fexpaT = 0.0
    tconc = 0.0 

999 format (4(1x,i3),5(1x,e10.4))
998 format (7(1x,i3),5(1x,e10.4))
!

    DO g=1,ngroup
      fact  = 0.0
      if (popf(g) /= 0.0) then

        rand = 0

        do isam = 1, amax


			Call randomr(arand2,idum2)
			Call randomr(arand2,idum2)
		! Start of random value selection for micro factors for a home tract
			DO s1=1, nmicro
				DO h1=1, hblock
					if(penA(1,s1)==1 .OR. penA(1,s1)==2 .OR. penA(1,s1) ==3 .OR. penA(1,s1) == 4) then
					    penA1= int(penA(1,s1))
						penA2= penA(2,s1)
						penA3= penA(3,s1)
						penA4= penA(4,s1)
						penL = penA(5,s1)
						penU = penA(6,s1)
						call Distribution(penA1,penA2,penA3,penA4,penX,idum2,2,penL,penU)
						if (penX==-9999.0000 .OR. penX>1 .OR. penX<0) then
							write(*,*) ' Penetration factors are not properly specified'
							stop
						else
							pen(s1,h1)=penX
						end if
					else if (penA(1,s1)==5) then 
						do s3=1,int(penA(2,s1))
						s4=s3+2
						DataArray(s3)=penA(s4,s1)
						end do
					call Dataset(DataArray,int(penA(2,s1)),pen(s1,h1),idum2)
					else
					write(*,*)' Error in reading the factors file'
					end if 
				END DO
			END DO 

			AddP=0									!Add for home tract
			if(Yproduct==0) then
			DO ipt=1,npt
				Do s1=1, nmicro
					Call randomr(arand2,idum2)
!					write(*,*) arand2
					if (arand2<prodfra2(ipt,s1)) then
!					    write(*,*) ipt,s1,arand2,prodfra2(ipt,s1)
						Call random2(ProSel,idum2, 10)
						Do h1=1,hblock
							AddP(s1,h1)=AddP(s1,h1)+Prodconc2(ipt,s1,h1,ProSel)
						End do
					end if
				end do
			End do
			end if

			AddM=0
			if(YM1==0 .OR. YM2==0) then
			Do imt=1,nmt1
				Do s1=1,nmicro
					Call randomr(arand2,idum2)
!					write(*,*) arand2
						if(arand2<m1fra2(imt,s1)) then
!							write(*,*) imt,s1,arand2,m1fra2(imt,s1)
							Call random2(ProSel,idum2, 10)
							Do h1=1,hblock
								AddM(s1,h1)=AddM(s1,h1)+m1conc2(imt,s1,h1,ProSel)
							end do
						else if(arand2<(m1fra2(imt,s1)+m2fra2(imt,s1))) then
!						    write(*,*) imt,s1,arand2,m1fra2(imt,s1)+m2fra2(imt,s1),m2fra2(imt,s1)
							Call random2(ProSel,idum2, 10)
							Do h1=1,hblock
								AddM(s1,h1)=AddM(s1,h1)+m2conc2(imt,s1,h1,ProSel)
							end do
						else
						    Call random2(ProSel,idum2, 10)
							Do h1=1,hblock
								AddM(s1,h1)=AddM(s1,h1)+0.0
							end do
						end if
				end do
			End do		
			end if
			
			AddC=0
			if(Ycomb==0) then
			DO ict=1,nct
				Do s1=1, nmicro
					Call randomr(arand2,idum2)
!					write(*,*) arand2
					if (arand2<combfra2(ict,s1)) then
!					    write(*,*) ipt,s1,arand2,prodfra2(ipt,s1)
						Call random2(ProSel,idum2, 10)
						Do h1=1,hblock
							AddC(s1,h1)=AddC(s1,h1)+combconc2(ict,s1,h1,ProSel)
						End do
					end if
				end do
			End do
			end if

			ADDT=0
			if(Yauto==0) then
			Do itt = 1, 1
				Do s1=1,nmicro
				    Call random2(ProSel,idum2,10)
					Do h1=1,hblock						
							AddT(s1,h1)=AddT(s1,h1)+autoconc2(itt,s1,h1,ProSel)
					End do
				End do 
			End do 
			End if
			
!write(*,*) '1'
			zoneidxh=1
			DataArray=0
			DO s1=1, nmicro				! Proximity Factor Selection- Begin
				DO s=1, nsource

				  findnm=0

				  DO itq=1, nmsource					! check if the source is an on-road mobile source
					if(nmobiles(itq)==s) then
						findnm=1
						exit
					end if
				  end do								! end of check

				 if(findnm==1) then				    !begin of mobile sources
					Call randomr(arand2,idummb1)
					DISTHTOT=0.0
					findist=0

					Do itq1=1, maxring
						if(commut==2) then
						DISTHTOT=DISTHTOT+DISTH(itq1,g,jc)
						else if (commut==1) then
						DISTHTOT=DISTHTOT+DISTHA(itq1,g,jc)
						end if
						if(itq1==3 .and. DISTHTOT<1.0) then
						DISTHTOT=1.0
						end if
						if(DISTHTOT>=arand2) then
						findist=itq1
						zoneidxh(s1,s)=itq1
						exit
						end if 
					End do
										
					if(proxA(1,s,s1,findist)==1 .OR. proxA(1,s,s1,findist)==2 .OR. proxA(1,s,s1,findist) ==3 .OR. proxA(1,s,s1,findist) ==4) then
						proxA1= int(proxA(1,s,s1,findist))
						proxA2= proxA(2,s,s1,findist)
						proxA3= proxA(3,s,s1,findist)
						proxA4= proxA(4,s,s1,findist)
						proxL = proxA(5,s,s1,findist)
						proxU = proxA(6,s,s1,findist)

						call Distribution(proxA1,proxA2,proxA3,proxA4,proxX,idum2,3,proxL,proxU)
						if(proxX==-9999.0000 .OR. proxX <1) then
							write(*,*) 'Proximity factors are not properly specified'
							stop
						else 
							prox(s1,s)=proxX
						end if
					else if (proxA(1,s,s1,findist)==5) then 
						do s3=1, proxA(2,s,s1,findist)
						s4=s3+2
						DataArray(s3)=proxA(s4,s,s1,findist)
						end do  
					call Dataset(DataArray,int(proxA(2,s,s1,findist)),prox(s1,s),idum2)
					else
					write(*,*)' Error in reading the factors file'
					end if

!					else if (findist==maxring) then  ! in the far region D3 Ring

!						prox(s1,s)=1.0
			
!					end if 
					
					 
				  end if ! end of mobile sources

				  if(findnm==0) then ! begin of nonmobile sources
					if(proxA(1,s,s1,1)==1 .OR. proxA(1,s,s1,1)==2 .OR. proxA(1,s,s1,1) ==3 .OR. proxA(1,s,s1,1) ==4) then
						proxA1= int(proxA(1,s,s1,1))
						proxA2= proxA(2,s,s1,1)
						proxA3= proxA(3,s,s1,1)
						proxA4= proxA(4,s,s1,1)
						proxL = proxA(5,s,s1,1)
						proxU = proxA(6,s,s1,1)
						call Distribution(proxA1,proxA2,proxA3,proxA4,proxX,idum2,3,proxL,proxU)
						if(proxX==-9999.0000 .OR. proxX <1) then
							write(*,*) 'Proximity factors are not properly specified'
							stop
						else 
							prox(s1,s)=proxX
						end if
					else if (proxA(1,s,s1,1)==5) then 
						do s3=1, proxA(2,s,s1,1)
						s4=s3+2
						DataArray(s3)=proxA(s4,s,s1,1)
						end do  
					call Dataset(DataArray,int(proxA(2,s,s1,1)),prox(s1,s),idum2)
					else
					write(*,*)' Error in reading the factors file'
					end if
				  end if ! end of non mobile sources	 

				END DO
			END DO		!Proximity Factor Selection- End

			1511 format(I2,I3,37F8.3)
!write(*,*) '10'
			if (jtest==1 .AND. isam<6) then
				do h1=1, hblock
					write(151,1511) jtest,isam, addP(:,h1)
					write(151,1511) jtest,isam, addM(:,h1)
					write(151,1511) jtest,isam, addC(:,h1)
					write(151,1511) jtest,isam, addT(:,h1)
					write(152,1511) jtest,isam, pen(:,h1)
				end do 
				do s=1, nsource
					write(153,1511) jtest,isam, prox(:,s)
				end do 
			end if 
			! End of random value selection for micro-factors for a home tract		

			!
			! Random Selection of Air Quality Dataset
			Call randomr(arand3,idum3)
			Call randomr(arand3,idum3)
			Call randomr(arand3,idum3)

			Call random2(AirSel,idum3, AirNo)
			ambh2(:,:) = ambh(:,:,AirSel)
			amback2 = amback(AirSel)

			! Calculate background concentration for each time step and micro environment
			!
			!write(*,1551) 'end of factors'
			!1551 format('+','processed',A30)
			do m = 1, nmicro
				do b = 1, hblock
					ii = m+nmicro*(b-1)
					concb(ii,1) = pen(m,b)*(bconc+amback2)
					concaddP(ii,1)=addP(m,b)
					concaddM(ii,1)=addM(m,b)
					concaddC(ii,1)=addC(m,b)
					concaddT(ii,1)=addT(m,b)
				end do
			end do


			if(nmsource>0) then

	            if(commut==2) then
				Cterm = MAX(0.000001, AREAH(2,jc)+AREAH(3,jc))
				Afar = AREAH(3,jc)/Cterm
				A2 = AREAH(2,jc)/Cterm

				if(Cterm == 0.0) then
				write(*,*) 'cterm=0',' ',g,' ',jc
				stop
				end if

				else if (commut==1) then
					Cterm = MAX(0.000001, AREAHA(2,jc)+AREAHA(3,jc))
					Afar = AREAHA(3,jc)/Cterm
					A2 = AREAHA(2,jc)/Cterm
				end if
	
				Do k=1, nmsource

					Do b=1, hblock
						
						ambh2(b,nmobiles(k)) =  ambh2(b,nmobiles(k))/(Afar+A2*RD2(k))

					End do 

				End do

			End if 

!if(g==3 .and. isam==1) then

!write(*,*) ctractid(jc),' ',jc,Afar,a2,' ',cterm

!end if


			amblh = CSHIFT(ambh2,-1,1)

			conc(:,1,:) = 0.0
				DO s=1,nsource
					DO m=1,nmicro
				        DO b=1,hblock
							i=m+nmicro*(b-1)
								IF (lag(m,s,zoneidxh(m,s))==0) THEN               
								conc(i,1,s)=prox(m,s)*pen(m,b)*ambh2(b,s)
								ELSE                                            
								conc(i,1,s)=prox(m,s)*pen(m,b)*amblh(b,s)
								END IF
						END DO
					END DO 
				END DO


! end of concentration calculation for home tract
          arand = 0.0
          call randomr(arand,idum)

          arand4 = 0.0
          call randomr4(arand4,idum4)

		  dcomSel=0
		  dcomtotal=0.0

		  Do dcom=1, ncom
			dcomtotal=dcomtotal+fracpop(dcom,g,jc)
			
			if(dcom==ncom .and. dcomtotal/=1.0) dcomtotal=1.0
			if(dcomtotal>=arand4) then
				dcomSel=dcom
				exit
			end if

		  END DO
		  
		  if(dcomSel==0) then
		  
		  write(*,*)'wrong in fracpop file'
		  stop
		  
		  end if
	
		  if(dcomSel==1) then
			commutnew = 1
		  else if (dcomSel==2) then
		    commutnew = 2
		  else
		  
		  write(*,*) ' Error with commuting mode selection'
		  stop
		  end if
		  	  
  		do t=1,ntype  ! removable 9/23/05
		do cls=1,clusno(g,t,dcomSel)
				if(ival(g,t,dcomSel,cls)==0) then
!				ifind=ifind+1
				dcomSel=1
				commutnew = 1
				write(*,*) 'Error: zero CHAD records for','g=',g,' t=',t,' com=',dcomSel
				stop
				end if
		end do 
		end do ! removable 9/23/05
! if a person is a commuting, then determine if this person public transit (dcar=1) or take car (dcar=2) to work
		dcarSel=0
		if (dcomSel==2) then
		  arand5 = 0.0
		  call randomr4(arand5,idum4)
		  dcartotal=0.0
		  do dcar=1,ncar
			dcartotal=dcartotal+compro(dcar,jc)
			if(dcar==ncar .and. dcartotal/=1.0) dcartotal=1.0
			if(dcartotal >=arand5) then
				dcarSel=dcar
				exit
			end if
		  end do
		end if


!write(9000,*) idum, ' ','a'

! begin of concentration for work tract
		   kcdumy=0
          IF (commutnew == 2) THEN        !if there is commuting find exposure in work tracts
           conc(:,2,:) = 0.0
           ambw(:,:,:) = 0.0
           amblw(:,:) = 0.0
		   ambackw(:) =0.0
		   ambackw2=0
           ifnd = 0
           ifnd1 = 0
           ifnd2 = 0
		   
!           if (jcom(g,1) == 1.or.jcom(g,2) == 1 .or.jcom(g,3) == 1) then
            do kc = first(jc), last(jc)
              if (frworkers(kc) >= arand) then
			    kcdumy=kc
                do ii = 1, maxst
                  if (bstind(ii) == wtractid(kc)(1:2)) then
                    ifnd2 = ii
                    exit
                  end if
                end do
                if (ifnd2 == 0) then
                  write (60,*) 'airtract match with worktract not found'
                  ambw2 = ambh2
				  ambackw2=amback2
                  exit
                end if
                do ii = stsump(ifnd2),stsuma(ifnd2)
                  if (ftractid(ii) == wtractid(kc)(1:5)) then
                    ifnd = ii
                    exit
                  end if
                end do
                if (ifnd == 0) then
                  write (60,*) 'airtract match with worktract not found'
                  ambw2 = ambh2
				  ambackw2=amback2
                  exit
                end if
                do ii = psump(ifnd), asuma(ifnd)
                  if (atractid(ii) == wtractid(kc)) then
                    ifnd1 = ii
					n11=0
					do i3=first1(ii),last1(ii)
					n11=n11+1
                    ambw(:,:,n11) = aqout(:,:,i3)
					ambackw(n11) = backC(i3)
					end do 
					AirNo2 = n11
					Call random2(AirSel,idum3, AirNo2)
					ambw2(:,:) = ambw(:,:,AirSel)
					ambackw2 = ambackw(AirSel)
                    exit
                  end if
                end do
                if (ifnd1 == 0) then
                  write (60,*) 'airtract match with worktract not found'
                  ambw2 = ambh2
				  ambackw2=amback2
                  exit
                end if
                exit
              end if
            end do
!          end if

!		Start of random value selection of work tract
			DO s1=1, nmicro
				DO h1=1, hblock
					if(penA(1,s1)==1 .OR. penA(1,s1)==2 .OR. penA(1,s1) ==3 .OR. penA(1,s1) ==4) then
						penA1= int(penA(1,s1))
						penA2= penA(2,s1)
						penA3= penA(3,s1)
						penA4= penA(4,s1)
						penL = penA(5,s1)
						penU = penA(6,s1)
						call Distribution(penA1,penA2,penA3,penA4,penX,idum2,2,penL,penU)
						if (penX==-9999.0000 .OR. penX>1 .OR. penX<0) then
							write(*,*) ' Penetration factors are not properly specified'
							stop
						else
							pen(s1,h1)=penX
						end if
					else if (penA(1,s1)==5) then 
						do s3=1, penA(2,s1)
						s4=s3+2
						DataArray(s3)=penA(s4,s1)
						end do 
					call Dataset(DataArray,int(penA(2,s1)),pen(s1,h1),idum2)
					else
					write(*,*)' Error in reading the factors file'
					end if 
					
				END DO
			END DO 

			AddP=0									! Add factor for a work tract
			if(Yproduct==0) then
			DO ipt=1,npt
				Do s1=1, nmicro
					Call randomr(arand2,idum2)
					if (arand2<prodfra2(ipt,s1)) then
					    Call random2(ProSel,idum2, 10)
						Do h1=1,hblock
							
							AddP(s1,h1)=AddP(s1,h1)+Prodconc2(ipt,s1,h1,ProSel)
						End do
					end if
				end do
			End do
			end if

			addM=0
		
			if(YM1==0 .OR. YM2==0) then
			Do imt=1,nmt1
				Do s1=1,nmicro
					Call randomr(arand2,idum2)
						if(arand2<m1fra2(imt,s1)) then
						    Call random2(ProSel,idum2, 10)
							Do h1=1,hblock
								
								AddM(s1,h1)=AddM(s1,h1)+m1conc2(imt,s1,h1,ProSel)
							end do
						else if(arand2<(m1fra2(imt,s1)+m2fra2(imt,s1))) then
						    Call random2(ProSel,idum2, 10)
							Do h1=1,hblock
								
								AddM(s1,h1)=AddM(s1,h1)+m2conc2(imt,s1,h1,ProSel)
							end do
						else
						    Call random2(ProSel,idum2, 10)
							Do h1=1,hblock
								
								AddM(s1,h1)=AddM(s1,h1)+0.0
							end do
						end if
				end do
			End do		
			end if

			AddC=0
			if(YComb==0) then
			DO ict=1,nct
				Do s1=1, nmicro
					Call randomr(arand2,idum2)
!					write(*,*) arand2
					if (arand2<combfra2(ict,s1)) then
!					    write(*,*) ipt,s1,arand2,prodfra2(ipt,s1)
                        Call random2(ProSel,idum2, 10)
						Do h1=1,hblock
							
							AddC(s1,h1)=AddC(s1,h1)+combconc2(ict,s1,h1,ProSel)
						End do
					end if
				end do
			END DO
			end if 

			ADDT=0
			if(Yauto==0) then
			Do itt = 1, 1
				Do s1=1,nmicro
				    Call random2(ProSel,idum2,10)
					Do h1=1,hblock
						
							AddT(s1,h1)=AddT(s1,h1)+autoconc2(itt,s1,h1,ProSel)
					End do
				End do 
			End do 
			End if
			

			zoneidxw=1

			DataArray=0
			DO s1=1, nmicro
				DO s=1, nsource

												
				  findnm=0

				  DO itq=1, nmsource					! check if the source is an on-road mobile source
					if(nmobiles(itq)==s) then
						findnm=1
						exit
					end if
				  end do								! end of check

				 if(kcdumy==0) findnm=0					! if it is not a commuting group, no kc will be selected- also removable because kcdumy will always be selected

				 if(findnm==1) then				    !begin of mobile sources
					Call randomr(arand2,idummb1)
					DISTHTOT=0.0
					findist=0
					Do itq1=1, maxring
						DISTHTOT=DISTHTOT+DISTW(itq1,g,kcdumy)
						if(itq1==3 .and. DISTHTOT<1.0) then
						DISTHTOT=1.0
						end if
						if(DISTHTOT>=arand2) then
						findist=itq1
						zoneidxw(s1,s)=itq1
						exit
						end if 
					End do
					
					if(proxA(1,s,s1,findist)==1 .OR. proxA(1,s,s1,findist)==2 .OR. proxA(1,s,s1,findist) ==3 .OR. proxA(1,s,s1,findist) ==4) then
						proxA1= int(proxA(1,s,s1,findist))
						proxA2= proxA(2,s,s1,findist)
						proxA3= proxA(3,s,s1,findist)
						proxA4= proxA(4,s,s1,findist)
						proxL = proxA(5,s,s1,findist)
						proxU = proxA(6,s,s1,findist)

						call Distribution(proxA1,proxA2,proxA3,proxA4,proxX,idum2,3,proxL,proxU)
						if(proxX==-9999.0000 .OR. proxX <1) then
							write(*,*) 'Proximity factors are not properly specified'
							stop
						else 
							prox(s1,s)=proxX
						end if
					else if (proxA(1,s,s1,findist)==5) then 
						do s3=1, proxA(2,s,s1,findist)
						s4=s3+2
						DataArray(s3)=proxA(s4,s,s1,findist)
						end do  
					call Dataset(DataArray,int(proxA(2,s,s1,findist)),prox(s1,s),idum2)
					else
					write(*,*)' Error in reading the factors file'
					end if

!					else if (findist==maxring) then  ! in the far region D3 Ring

!						prox(s1,s)=1.0
			
!					end if 
					
					 
				  end if 
				  
! end of mobile sources


!end of new code    
					if(findnm==0) then					!begin of non on-road mobile source

					if(proxA(1,s,s1,1)==1 .OR. proxA(1,s,s1,1)==2 .OR. proxA(1,s,s1,1) ==3 .OR. proxA(1,s,s1,1) ==4) then
					    proxA1= int(proxA(1,s,s1,1))
						proxA2= proxA(2,s,s1,1)
						proxA3= proxA(3,s,s1,1)
						proxA4= proxA(4,s,s1,1)
						proxL = proxA(5,s,s1,1)
						proxU = proxA(6,s,s1,1)
						call Distribution(proxA1,proxA2,proxA3,proxA4,proxX,idum2,3,proxL,proxU)
						if(proxX==-9999.0000 .OR. proxX <1) then
							write(*,*) 'Proximity factors are not properly specified'
							stop
						else 
							prox(s1,s)=proxX
						end if
					else if (proxA(1,s,s1,1)==5) then 
						do s3=1, proxA(2,s,s1,1)
						s4=s3+2
						DataArray(s3)=proxA(s4,s,s1,1)
						end do 
					call Dataset(DataArray,int(proxA(2,s,s1,1)),prox(s1,s),idum2)
					else
					write(*,*)' Error in reading the factors file'
					end if 

					end if ! end of non on-road mobile sources
				END DO
			END DO


			if (jtest==1 .AND. isam<6) then
				do h1=1, hblock
					write(151,1511) jtest, isam, addP(:,h1)
					write(151,1511) jtest, isam, addM(:,h1)
					write(151,1511) jtest, isam, addC(:,h1)
					write(152,1511) jtest, isam, pen(:,h1)
				end do 
				do s=1, nsource
					write(153,1511) jtest, isam, prox(:,s)
				end do
			end if 
			! End of random value selection for microfactors for a work tract
			!
			! Calculate background concentration for each time step and micro environment
			!
			do m = 1, nmicro
				do b = 1, hblock
					ii = m+nmicro*(b-1)
					concb(ii,2) = pen(m,b)*(bconc+ambackw2)
					concaddP(ii,2) = addP(m,b)
					concaddM(ii,2) = addM(m,b)
					concaddC(ii,2) = addC(m,b)
					concaddT(ii,2) = addT(m,b)
				end do
			end do
			!
			if(nmsource>0) then

				if(kcdumy/=0) then     ! (removable) if there is no worktract, ambw2 is zero and no need to do correction
				
				Cterm = MAX(0.000001, AREAW(2,kcdumy)+AREAW(3,kcdumy))
				Afar = MAX(0.000001, AREAW(3,kcdumy)/Cterm)
				A2 = MAX(0.000001, AREAW(2,kcdumy)/Cterm)
				
				if(Cterm==0) then

					write(*,*) 'error in tract distance probability file ',kcdumy
					stop

				end if

				if (Afar == 0.0 .and. A2==0.0) then

					write(*,*) 'error in tract distance probability file ',kcdumy
					stop

				end if



				Do k=1, nmsource

					Do b=1, hblock
						
						ambw2(b,nmobiles(k)) =  ambw2(b,nmobiles(k))/(Afar+A2*RD2(k))

					End do 

				End do

				end if 

			End if

            amblw = CSHIFT(ambw2,-1,1)



          do s = 1, nsource
            DO m=1,nmicro
              DO b=1,hblock
                ii=m+nmicro*(b-1)
                IF (lag(m,s,zoneidxw(m,s))==0) THEN               
                  conc(ii,2,s)=prox(m,s)*pen(m,b)*ambw2(b,s)
                ELSE                                            
                  conc(ii,2,s)=prox(m,s)*pen(m,b)*amblw(b,s)
                END IF
              END DO
            END DO
          end do ! end of source

! end of commut=2 ?
END IF	! end of calculation of work concentrations



! begin home tract code
!          arand4 = 0.0
!          call randomr(arand4,idum)		

		  tday = 0.0
          do t = 1, ntype
			ifind=0
			do cls=1,clusno(g,t,dcomSel)
				if(ival(g,t,dcomSel,cls)/=0) then 
				ifind=ifind+1
				end if
			end do 
			if (ifind>0) then
				tday = tday+bmax(t)
			end if
          end do

		  9001 format(296F6.0)
		  randcls=0
		  clsdays=0
		  dursc=0 ! new				

          DO t=1,ntype

		    ifind=0
			do cls=1,clusno(g,t,dcomSel)
				if(ival(g,t,dcomSel,cls)/=0) then
				ifind=ifind+1
!				ifind=0
!				write(*,*) 'Error: zero CHAD records for','g=',g,' t=',t,' com=',dcomSel
!				stop
				end if
			end do 

            if (ifind>0) then

				dursc2=0
				clsdays1=0
				
				call cluster(int(bmax(t)),clusno(g,t,dcomSel),maxcls,Freq(g,t,dcomSel,:),Prob(g,t,dcomSel,:,:),clsdays(t,:),idum)

!				write(*,*) int(bmax(t)),clusno(g,t,dcomSel),clsdays(t,:) 
				
				do cls=1,clusno(g,t,dcomSel)

					if (ival(g,t,dcomSel,cls)== 1) then
					randcls(t,cls)=1
					else	
					call random3(randcls(t,cls),idum,ival(g,t,dcomSel,cls))
					end if
!					dursc(:,1) = dursc(:,1)+dur(:,1,t,g,cls,randcls(t,cls))*clsdays(t,cls)
					dursc2(:,:) = dur(:,:,dcomSel,t,g,cls,randcls(t,cls))
		
					if(commutnew==2 .and. kcdumy/=0 .and. t/=3) then

						nvtotal=0
						nptotal=0
						nrtotal=0
					
					do m = 1, nmicro		! finding the total minutes of vehicles and non vehicle micros
						findv=0
						DO itq2=1, nmvehicle
							if(m==nvehicles(itq2)) then
							findv=1
							exit
							end if
						END DO

						DO itq3=1,nmpublict
							if(m==npublict(itq3)) then
							findv=2
							exit
							end if
						END DO
						
						if(findv==0) then 	
						do b = 1, hblock
							ii = m+nmicro*(b-1)
							   do icom=1, commut
								nrtotal=nrtotal+dursc2(ii,icom)	
							   end do 
						end do
						else if (findv==1) then
						do b = 1, hblock
							ii = m+nmicro*(b-1)
							   do icom=1, commut
							   nvtotal=nvtotal+dursc2(ii,icom)	
							   end do 
						end do
						else if (findv==2) then
						do b = 1, hblock
							ii = m+nmicro*(b-1)
							   do icom=1, commut
							   nptotal=nptotal+dursc2(ii,icom)	
							   end do 
						end do
                        end if
					end do 					! end of finding the total minutes of vehicles and non vehicle micros
!				write(*,*) nvtotal,nptotal,nrtotal,sum(dursc2),dcarSel,comtime(dcarSel,jc),distfra(kcdumy)
					if(nrtotal/=0) then  ! prevent the chad with 1440 minutes in commuting vehicles from interfering calculation
					if(nvtotal/=0 .or. nptotal/=0) then ! begin of correcting time in  micros
					do m = 1, nmicro	!	begin of correcting time in non-vehicle micros
						findv=0
						DO itq2=1, nmvehicle
							if(m==nvehicles(itq2)) then
							findv=1
							exit
							end if
						END DO

						DO itq3=1,nmpublict
							if(m==npublict(itq3)) then
							findv=2
							exit
							end if
						END DO
						
						if(findv==1 .or. findv==2) cycle
								
						if(findv==0) then 	

						do b = 1, hblock
							ii = m+nmicro*(b-1)
							   do icom=1, commut
									dursc2(ii,icom)	=dursc2(ii,icom)/nrtotal*(1440-comtime(dcarSel,jc)*distfra(kcdumy))	
							   end do 
						end do
						end if
					end do	! end of correcting time in non-vehicle micros				

					if(dcarSel==1) then ! public transit
						findcax=0
						if(nptotal /=0.0 .and. nvtotal==0.0) then
						  findcax=1
						else if(nptotal ==0.0 .and. nvtotal/=0) then
						  findcax=2
						else if(nptotal/=0 .and. nvtotal/=0) then
						  findcax=3
						else
						write(*,*) 'error 1 in commuting time adjustment'
						stop
						end if


						if(findcax==1 .or. findcax==3) then
							
							Do itq3=1,nmpublict
							  Do b=1,hblock
								ii = npublict(itq3)+nmicro*(b-1)
							   do icom=1, commut
									dursc2(ii,icom)=dursc2(ii,icom)/nptotal*comtime(dcarSel,jc)*distfra(kcdumy)
							   end do
							  End do 
							End do

							if(findcax==3) then

							 Do itq2=1, nmvehicle
								Do b=1, hblock
									ii=nvehicles(itq2)+nmicro*(b-1)
									do icom=1,commut
										dursc2(ii,icom)=0.0
									end do
								end do 
							 End do

							 end if



						end if

						if(findcax==2) then

							Do b=1, hblock
							  do icom=1,commut
								blocksum=0.0
								Do itq2=1, nmvehicle			!check if which block has time
									ii = nvehicles(itq2)+nmicro*(b-1)	
									blocksum=blocksum+dursc2(ii,icom)
									dursc2(ii,icom)=0.0
								End do 
								
								if(blocksum/=0) then
									Do itq3=1,nmpublict
										ii = npublict(itq3)+nmicro*(b-1)
										dursc2(ii,icom)=blocksum/nvtotal/nmpublict*comtime(dcarSel,jc)*distfra(kcdumy)
									end do
								end if
							  end do 
                            End do
						end if

					else if(dcarSel==2) then ! cars/trucks

						findcax=0	
						if(nptotal ==0.0 .and. nvtotal/=0.0) then
						  findcax=1
						else if(nptotal/=0.0 .and. nvtotal==0) then
						  findcax=2
						else if(nptotal/=0.0 .and. nvtotal/=0) then
						  findcax=3
						else
						write(*,*) 'error 2 in commuting time adjustment'
						stop
						end if
						
						if(findcax==1 .or. findcax==3) then
							
							Do itq2=1,nmvehicle
							  Do b=1,hblock
								ii = nvehicles(itq2)+nmicro*(b-1)
							   do icom=1, commut
									dursc2(ii,icom)=dursc2(ii,icom)/nvtotal*comtime(dcarSel,jc)*distfra(kcdumy)
							   end do
							  End do 
							End do
							
							if(findcax ==3) then	! set the minutes to zero in public transit micros

							Do itq3=1, nmpublict
							  Do b=1,hblock
								ii=npublict(itq3)+nmicro*(b-1)
								do icom=1,commut
									dursc2(ii,icom)=0.0
								end do
							 End do
							End do
						    
							end if								
	
						end if

						if(findcax==2) then

							Do b=1, hblock
							  do icom=1,commut
								blocksum=0.0
								Do itq3=1, nmpublict			!check if which block has time
									ii = npublict(itq3)+nmicro*(b-1)	
									blocksum=blocksum+dursc2(ii,icom)
									dursc2(ii,icom)=0.0
								End do 
								
								if(blocksum/=0) then
									Do itq2=1,nmvehicle
										ii = nvehicles(itq2)+nmicro*(b-1)
										dursc2(ii,icom)=blocksum/nptotal/real(nmvehicle)*comtime(dcarSel,jc)*distfra(kcdumy)
										
									end do
								end if
							  end do 
                            End do
						end if
					
					else

						write(*,*) 'error 3 in commuting time adjustment = dcarSel=0'
						stop
					End if ! end of the commute time adjustment for public transit or vehicle micros

					end if		! end of time correction
					end if      ! end of if nrtotal==0 minutes

						if(abs(sum(dursc2)-1440.0)>0.3) then
						write(*,*) 'Error: the corrected CHAD record > 1440.0 min.'
						stop
						end if
	
					end if   ! end of the whole code for commuting time adjustment

		
				    dursc(:,:,t) = dursc(:,:,t)+dursc2(:,:)*clsdays(t,cls)


				end do 

				clsdays1=clsdays(t,:)
				dursc(:,:,t) =dursc(:,:,t)/(sum(clsdays1))


				fact(t) = (bmax(t)/(tday*1440.0))

			  do s = 1, nsource
                dexp1=DOT_PRODUCT(conc(:,1,s),dursc(:,1,t))

                cexp(g,s,isam) = cexp(g,s,isam)+dexp1*fact(t)
              end do
				
              dexpb=DOT_PRODUCT(concb(:,1),dursc(:,1,t))
			  dexpaP=DOT_PRODUCT(concaddP(:,1),dursc(:,1,t))
			  dexpaM=DOT_PRODUCT(concaddM(:,1),dursc(:,1,t))
			  dexpaC=DOT_PRODUCT(concaddC(:,1),dursc(:,1,t))
			  dexpaT=DOT_PRODUCT(concaddT(:,1),dursc(:,1,t))
              fexp(g,isam) = fexp(g,isam)+dexpb*fact(t)
			  fexpaP(g,isam) = fexpaP(g,isam)+dexpaP*fact(t)
			  fexpaM(g,isam) = fexpaM(g,isam)+dexpaM*fact(t)
			  fexpaC(g,isam) = fexpaC(g,isam)+dexpaC*fact(t)
			  fexpaT(g,isam) = fexpaT(g,isam)+dexpaT*fact(t)

            end if
          end do

! end of home tract code


if(commutnew==2) then


            do t = 1, ntype

				ifind=0
				
				do cls=1,clusno(g,t,dcomSel)
					if(ival(g,t,dcomSel,cls)/=0) then
						ifind=ifind+1
					end if
				end do 

				if (ifind>0) then
				
				  do s=1, nsource
				    dexp2=DOT_PRODUCT(conc(:,2,s),dursc(:,2,t))
					cexp(g,s,isam)=cexp(g,s,isam)+dexp2*fact(t)
				  end do     ! end source loop
				
!				write(9002,*) g,t,dcomSel,sum(dursc(:,:,t)),sum(dursc(:,2,t))

				  dexpb=DOT_PRODUCT(concb(:,2),dursc(:,2,t))
				  dexpaP=DOT_PRODUCT(concaddP(:,2),dursc(:,2,t))
				  dexpaM=DOT_PRODUCT(concaddM(:,2),dursc(:,2,t))
				  dexpaC=DOT_PRODUCT(concaddC(:,2),dursc(:,2,t))
			      dexpaT=DOT_PRODUCT(concaddT(:,2),dursc(:,2,t))
                  fexp(g,isam) = fexp(g,isam)+dexpb*fact(t)
			      fexpaP(g,isam) = fexpaP(g,isam)+dexpaP*fact(t)
			      fexpaM(g,isam) = fexpaM(g,isam)+dexpaM*fact(t)
			      fexpaC(g,isam) = fexpaC(g,isam)+dexpaC*fact(t)	
		          fexpaT(g,isam) = fexpaT(g,isam)+dexpaT*fact(t)


                end if ! there are nonzero number of chad entries
            end do   ! day type loop
			

		END IF   ! original end of commut2

        end do       ! end of isam loop

        do isam = 1, amax
          do s = 1, nsource
            tconc(g,isam) = tconc(g,isam)+cexp(g,s,isam)
          end do   ! source loop
          tconc(g,isam) = tconc(g,isam)+fexp(g,isam)+fexpaP(g,isam)+fexpaM(g,isam)+fexpaC(g,isam)+fexpaT(g,isam)
        end do     ! replication loop
      end if       ! population for that group is not zero
    end do         ! finished group loop
!
    if (commut == 2) then 
      do g = 1, ngroup
        do isam = 1, amax
          write (61,102) ctractid(jc)(1:2),ctractid(jc)(3:5),&
&ctractid(jc)(6:11),g,popf(g),(cexp(g,s,isam),s=1,nsource),fexp(g,isam),fexpaP(g,isam),fexpaM(g,isam),fexpaC(g,isam),fexpaT(g,isam),tconc(g,isam)
        end do
      end do
    else 
      do g = 1, ngroup
        do isam = 1, amax
          write (61,102) atractid(jc)(1:2),atractid(jc)(3:5),&
&atractid(jc)(6:11),g,popf(g),(cexp(g,s,isam),s=1,nsource),fexp(g,isam),fexpaP(g,isam),fexpaM(g,isam),fexpaC(g,isam),fexpaT(g,isam),tconc(g,isam)
        end do
      end do
    end if 
!

  END DO         ! Finshed tract
  close (61)

end do           ! finished states
102 format (1x,a,1x,a,1x,a,1x,i2,1x,f7.3,20(1x,e10.4))
104 format (a,1x,i6,1x,i6,1x,a,1x,i2)
!
!   Wrapup section
!
CALL SYSTEM_CLOCK(cl3,cr,cm)                              ! Job finishing time
WRITE(18,*) 'HAPEM Start time =',cl1,cr,cm                ! start time on log
WRITE(18,*) 'HAPEM Allocation =',cl2                      ! mid time on log
WRITE(18,*) 'There were ',ntrac14,' tracts in the study area.'       ! # of tracts
WRITE(18,*) 'HAPEM End time   =',cl3                      ! Put end time on log
WRITE(18,*) 'HAPEM Job time   =',REAL(cl3-cl1)/cr         ! Job execution time
CLOSE(18)                                                 ! Close log
DEALLOCATE(ambh,amblh,ambw,amblw,prox,addP,addM,addC,pen,lag,dur,conc,ival)  
DEALLOCATE(pop,popf,cexp)                                 ! Free dynamic arrays
STOP                                                      ! Stop execution
END PROGRAM hapem8                                        ! End of source code
!
!
!
SUBROUTINE random (rand,idum,nsel)
!
! This generates a random number ranging between 1 and "nsel".
!  The program is adopted from Press et al.(1992,pp.271)
!
! Declarations
!
IMPLICIT NONE
INTEGER :: nsel
REAL, ALLOCATABLE :: x(:)
REAL    :: ran1,frac,anum 
INTEGER, INTENT(OUT) :: rand
INTEGER, INTENT(IN OUT) :: idum
INTEGER :: i, k, init
!
!  init=nsel*50
   init=10
   ALLOCATE(x(init))
!
  x = 0.0
!
  do i = 1, init
    x(i) = ran1(idum)
  end do
!
  k = 0
  do i = 1, init
    anum=x(i)*nsel
    frac = anum-int(anum)
    if (frac < 0.5) then
      rand = int(anum)
    else
      rand= int(anum)+1
    endif
    if (rand < nsel .and. rand > 1) exit
   end do
!
!
   DEALLOCATE(x)
END SUBROUTINE RANDOM

SUBROUTINE random2 (rand,idum,nsel)
!
! This subroutine is used as a random generator in the dataset subroutine.
!
! Declarations
!
IMPLICIT NONE
INTEGER :: nsel
REAL, ALLOCATABLE :: x(:)
REAL    :: ran1,frac,anum 
INTEGER, INTENT(OUT) :: rand
INTEGER, INTENT(IN OUT) :: idum
INTEGER :: i, k, init
!
!  init=nsel*50
   init=10
   ALLOCATE(x(init))
!
  x = 0.0
!
  do i = 1, init
    x(i) = ran1(idum)
  end do
!
  k = 0
  do i = 1, init
    anum=x(i)*nsel
    frac = anum-int(anum)
    if (frac < 0.5) then
      rand = int(anum)+1
    else
      rand= int(anum)+1
    endif
    if (rand <= nsel .and. rand >= 1) exit
   end do
!
!
  DEALLOCATE(x)
END SUBROUTINE RANDOM2
!
!
function ran1(idum)
!
     IMPLICIT NONE
     INTEGER, PARAMETER :: ntab=32,ia=16807,im=2147483647,iq=127773,ir=2836
     REAL, PARAMETER :: eps=1.2e-7
     INTEGER :: idum,ndiv=(1+(im-1)/ntab),j,k
     REAL    :: ran1, am=(1.0/im), rnmx=(1.0-eps)
     SAVE iv, iy
     INTEGER :: iv(ntab)=0, iy=0
!
     if (idum <= 0 .or. iy == 0.0) then
       idum = max(-idum,1)
       do 11 j = ntab+8,1,-1
          k = idum/iq
          idum = ia*(idum-k*iq)-ir*k
          if (idum < 0) idum = idum +im
          if (j <= ntab) iv(j) = idum
 11    continue
       iy = iv(1)
     end if
     k = idum/iq
     idum = ia*(idum-k*iq)-ir*k
     if (idum < 0) idum = idum+im
     j = 1+iy/ndiv
     iy = iv(j)
     iv(j) = idum
     ran1 = min(am*iy,rnmx)
     return
end function ran1
!
SUBROUTINE randomr (arand,idum)
!
! This program will generate a random number ranging from
! ??? and ???. The program is adopted from Press et al.(1992,pp.271)
!
! Declarations
!
IMPLICIT NONE
INTEGER :: i
REAL    :: ran1
REAL, INTENT(OUT) :: arand
INTEGER, INTENT(IN OUT) :: idum
!
arand = 0.0
!
arand= ran1(idum)
!
END SUBROUTINE RANDOMR


SUBROUTINE random3 (rand,idum,nsel)
!
! This program will generate a random number ranging 
! between 1 and "nsel". The program is adopted from Press et al.(1992,pp.271)
!
! Declarations
!
IMPLICIT NONE
INTEGER :: nsel
REAL, ALLOCATABLE :: x(:)
REAL    :: ran3,frac,anum 
INTEGER, INTENT(OUT) :: rand
INTEGER, INTENT(IN OUT) :: idum
INTEGER :: i, k, init
!
!  init=nsel*50
   init=10
   ALLOCATE(x(init))
!
  x = 0.0
!
  do i = 1, init
    x(i) = ran3(idum)
  end do
!
  k = 0
  do i = 1, init
    anum=x(i)*nsel
    frac = anum-int(anum)
    if (frac < 0.5) then
      rand = int(anum)
    else
      rand= int(anum)+1
    endif
    if (rand <= nsel .and. rand >= 1) exit
   end do
!
!
   DEALLOCATE(x)
END SUBROUTINE RANDOM3


function ran3(idum) ! for activity pattern only
!
     IMPLICIT NONE
     INTEGER, PARAMETER :: ntab=32,ia=16807,im=2147483647,iq=127773,ir=2836
     REAL, PARAMETER :: eps=1.2e-7
     INTEGER :: idum,ndiv=(1+(im-1)/ntab),j,k
     REAL    :: ran3, am=(1.0/im), rnmx=(1.0-eps)
     SAVE iv, iy
     INTEGER :: iv(ntab)=0, iy=0
!
     if (idum <= 0 .or. iy == 0.0) then
       idum = max(-idum,1)
       do 11 j = ntab+8,1,-1
          k = idum/iq
          idum = ia*(idum-k*iq)-ir*k
          if (idum < 0) idum = idum +im
          if (j <= ntab) iv(j) = idum
 11    continue
       iy = iv(1)
     end if
     k = idum/iq
     idum = ia*(idum-k*iq)-ir*k
     if (idum < 0) idum = idum+im
     j = 1+iy/ndiv
     iy = iv(j)
     iv(j) = idum
     ran3 = min(am*iy,rnmx)
     return
end function ran3


SUBROUTINE randomr4 (arand,idum)
!
! This program will generate a random number ranging from
! ??? and ???. The program is adopted from Press et al.(1992,pp.271)
!
! Declarations
!
IMPLICIT NONE
INTEGER :: i
REAL    :: ran4
REAL, INTENT(OUT) :: arand
INTEGER, INTENT(IN OUT) :: idum
!
arand = 0.0
!
arand= ran4(idum)
!
END SUBROUTINE RANDOMR4

function ran4(idum)
!
     IMPLICIT NONE
     INTEGER, PARAMETER :: ntab=32,ia=16807,im=2147483647,iq=127773,ir=2836
     REAL, PARAMETER :: eps=1.2e-7
     INTEGER :: idum,ndiv=(1+(im-1)/ntab),j,k
     REAL    :: ran4, am=(1.0/im), rnmx=(1.0-eps)
     SAVE iv, iy
     INTEGER :: iv(ntab)=0, iy=0
!
     if (idum <= 0 .or. iy == 0.0) then
       idum = max(-idum,1)
       do 11 j = ntab+8,1,-1
          k = idum/iq
          idum = ia*(idum-k*iq)-ir*k
          if (idum < 0) idum = idum +im
          if (j <= ntab) iv(j) = idum
 11    continue
       iy = iv(1)
     end if
     k = idum/iq
     idum = ia*(idum-k*iq)-ir*k
     if (idum < 0) idum = idum+im
     j = 1+iy/ndiv
     iy = iv(j)
     iv(j) = idum
     ran4 = min(am*iy,rnmx)
     return
end function ran4