PROGRAM indexpop7                                ! First line of Fortran 90 code
!
! This program lists the tractid and its serial number from the population
! data file and passes over the information to airqual subroutine.
!
! Program developed by ICF Consulting for EPA in June 2000.
! Latest revision by MLG on June 1, 2000
! Revised for matching loops by MH on June 19,2002
! Added GETARG to call file 10
! Indexpop7 finalized by ICF September, 25, 2014
!
! Files used
! 10 : param_file.1
!      PARAM file used for input constants & filenames
!      (also used by all other subprograms)
! 14 : census.txt
!      Population input data read from text file (used in HAPEM for header)
! 18 : log_file.txt.usa 
!      LOG file (also used by all other programs in the APPEND mode)
! 24 : census.da 
!      Direct access population file used by AIRQUAL subprogram
! 44 : census_direct.ind
!      Direct access population tract ID file used in COMMUTE and AIRQUAL
! 96 : census.county_tract_pop_range
!      Sequential file for indices of total number of population tracts 
!      for each county. This file is used in COMMUTE subprogram
! 97 : census.state_county_pop_range
!      Sequential file for indices of total number of population counties 
!      per state. This file is used in COMMUTE subprogram
! 98 : statefip.dat
!      ASCII file for two digits state FIPs with their serial numbers 
!      Also used by COMMUTE and AIRQUAL subprograms
! 99 : counter.dat (This ASCII file is used in all subprograms)
!      This file has total number of chad entries in the input and output files
! 
! Declarations section
!
IMPLICIT NONE                                       ! Declare all variables
INTEGER, PARAMETER   :: maxst = 53                  ! Maximum # of states
INTEGER, ALLOCATABLE :: pop(:)                      ! Population of dem. groups
INTEGER :: ngroup                                   ! Max # of classified group
INTEGER :: acount                                   ! Max # of counties in census file
INTEGER :: first,last								! indices of counties
INTEGER :: sumcount(maxst)                          ! # of counties/state in census file
INTEGER :: cl1,cl2,cr,cm, varint                    ! Clock variables
INTEGER :: region1, region2                         ! Serial # of states be included
INTEGER :: noinct									! # of census in a county
INTEGER :: ios=0,i,ij,ii,jj,stsuma,stsump,ist       ! Indices & counters
INTEGER :: ntract                                   ! Total # of census tracts 
CHARACTER (LEN=2)    :: stind(maxst)                 ! State FIP in string
CHARACTER (LEN=11)   :: tractid                      ! Census tract ID
CHARACTER (LEN=120)  :: paramline, varname, varchar, FNAME  ! PARAM input
CHARACTER (LEN=100)  :: f14,f18,f24,f44,f441,f99,f98      ! File names
CHARACTER (LEN=100)  :: f97,f96                      ! File names
CHARACTER (LEN=100)  :: f20,f201,f202,f30,f301,f302,f40,f401,f402  ! 
CHARACTER (LEN=5)  :: county1,county2				 ! County fips
! NEW DEFINTION
INTEGER :: ndist,dfirst,dlast
INTEGER :: ncomt,mfirst,mlast
INTEGER :: nfrac,ffirst,flast
INTEGER :: k,j,ik
CHARACTER (LEN=11):: distid,distid2,comtid,comtid2,fracid,fracid2
REAL :: comtime(2),compro(2)
REAL,allocatable :: distpro(:,:),fracpro(:,:),areapro(:)
!
! PARAMS section
!
CALL SYSTEM_CLOCK(cl1,cr,cm)                        ! Execution start time
CALL GETARG(1,FNAME)
OPEN (UNIT=10,FILE=FNAME,RECL=120,ACTION='READ')  
                                                    ! Open PARAMS file
!
DO                                                  ! Loop over records
   READ(10,'(A120)',IOSTAT=ios) paramline           ! Read one PARAMS line
   IF (ios/=0) EXIT                                 ! Exit on end of file
   i =INDEX(paramline,'=')                          ! Check for equal sign
   IF (i==0) CYCLE                                  ! Discard line if no '='
   varname=ADJUSTL(paramline(1:i-1))                ! Part on left of '='
   varchar=ADJUSTL(paramline(i+1:))                 ! Part on right of '='
   IF (INDEX(varname,'poll')>0) CYCLE               ! Skip pollutant name
   IF (INDEX(varname,'unit')>0) CYCLE               ! Skip pollutant units
   IF (INDEX(varname,'file')>0) THEN                ! For file names...
      IF (INDEX(varname,'log')>0)       f18=varchar ! Set log file name
      IF (INDEX(varname,'counter')>0)   f99=varchar ! Set counter file name
      IF (INDEX(varname,'statefip')>0)  f98=varchar ! Set statefip file name
      IF (INDEX(varname,'populat')>0)   f14=varchar ! Set population file name
	  IF (INDEX(varname,'DistToRoad')>0) f20=varchar ! Set distance_to_probability file
	  IF (INDEX(varname,'CommutTime')>0) f30=varchar  
	  IF (INDEX(varname,'CommutFrac')>0) f40=varchar			!CommutFrac
   ELSE                                             ! Else set parameters
      READ(varchar,*,IOSTAT=ios) varint             ! Convert to integer
      IF (ios/=0) CYCLE                             ! If not integer, discard
      IF (varname=='region1')        region1=varint ! first state for run
      IF (varname=='region2')        region2=varint ! last state for rn
      IF (varname=='ngroup')         ngroup =varint ! Set # demographic groups
	  IF (varname=='nreplic') then
	      write(*,*) ' Wrong parameter file.  Use P1.txt for this module'
		  stop
	  end if 
   END IF                                           ! Finished with line
END DO                                              ! Get next line
CLOSE(10)                                           ! Close PARAMS file
!
! Setup section
!
i = INDEX(f14,'.')-1                                    ! Locate extn on f12
IF (i==0) i=LEN_TRIM(f14)                               ! Trim f14 file name
f44 = f14(:i)//'_direct.ind'                            ! Name of index file
f441= f14(:i)//'_censusid.tmp'							! Name of census id temp file
i = INDEX(f14,'.')-1                                    ! Locate extn on f14
IF (i==0) i=LEN_TRIM(f14)                               ! Trim f14 file name
f24 = f14(:i)//'.da'                                    ! Direct access file
f96 = f14(:i)//'.county_tract_pop_range'                ! Direct access file
f97 = f14(:i)//'.state_county_pop_range'                ! Direct access file
i = INDEX(f20,'.')
f201=f20(:i)//'STIDX'
f202=f20(:i)//'dat'
i = INDEX(f30,'.')
f301=f30(:i)//'STIDX'
f302=f30(:i)//'dat'
i = INDEX(f40,'.')
f401=f40(:i)//'STIDX'
f402=f40(:i)//'dat'
!

allocate(distpro(3,ngroup),areapro(3),fracpro(2,ngroup))


! Main section
!
OPEN (UNIT=14,FILE=f14,RECL=36+8*ngroup,ACTION='READ')   ! Pop. input file
OPEN (UNIT=18,FILE=f18,POSITION='APPEND',ACTION='WRITE') ! Open log file
OPEN (UNIT=20,FILE=f20,ACTION='READ')					! set distance probability file
OPEN (UNIT=201,FILE=f201,ACTION='WRITE')					!
OPEN (UNIT=202,FILE=f202,ACCESS='DIRECT',RECL=15+3*ngroup*4, ACTION='WRITE')	!
OPEN (UNIT=30,FILE=f30,ACTION='READ')					! set distance probability file
OPEN (UNIT=301,FILE=f301,ACTION='WRITE')					!
OPEN (UNIT=302,FILE=f302,ACCESS='DIRECT',RECL=15+5*4, ACTION='WRITE')	!
OPEN (UNIT=40,FILE=f40,ACTION='READ')					! set distance probability file
OPEN (UNIT=401,FILE=f401,ACTION='WRITE')					!
OPEN (UNIT=402,FILE=f402,ACCESS='DIRECT',RECL=15+2*ngroup*4, ACTION='WRITE')	!
OPEN (UNIT=24,FILE=f24,ACCESS='DIRECT',RECL=4*ngroup)    ! Pop. output file
OPEN (UNIT=44,FILE=f44,ACCESS='direct', recl = 30)       ! Pop tractid file
OPEN (UNIT=441,FILE=f441,ACTION='READWRITE')				 ! Temp file
!OPEN (UNIT=96,FILE=f96,ACCESS='DIRECT',RECL=70)          ! Indices file
OPEN (UNIT=96,FILE=f96,ACTION='readwrite')
!OPEN (UNIT=97,FILE=f97,ACCESS='DIRECT',RECL=70)          ! Indices file
OPEN (UNIT=97,FILE=f97,ACTION='write')
OPEN (UNIT=98,FILE=f98,ACTION='READ')                    ! Open statefip file
OPEN (UNIT=99,FILE=f99,STATUS='OLD')                     ! Open counter file
WRITE(18,*) 'IndexPop Start time=',cl1,cr,cm           ! Execution start time
READ (14,*)                                            ! Discard header record
READ (14,*)                                            ! Skip header (commuting status)(+MD)
WRITE(18,*) 'Finished opening files'                   ! Message to log
!
! Load distance probability file

2011 format (I3,A3,3I8)
j=0
ndist=0
dfirst=0
dlast=0
do 

	read(20,*,iostat=ios) distid,areapro(:),(distpro(:,ik),ik=1,ngroup)
	if(ios/=0) exit
	ndist=ndist+1
!	write(*,*) '# of records processed = ',i
	dfirst=dlast+1
	k=1
	j=j+1
	write(202,rec=ndist) distid,areapro(:),(distpro(:,ik),ik=1,ngroup)

		do 
			read(20,*,iostat=ios) distid2,areapro(:),(distpro(:,ik),ik=1,ngroup)
			if(ios/=0) exit
			if(distid(1:2)==distid2(1:2)) then
			ndist=ndist+1
			k=k+1
			write(202,rec=ndist)distid2,areapro(:),(distpro(:,ik),ik=1,ngroup)
			else
			backspace(20)
			exit
			end if
		end do

	dlast=dlast+k
	write(201,2011) j,distid(1:2), dfirst,dlast

end do

!write(*,*) ndist
j=0
ncomt=0
mfirst=0
mlast=0
do 

	read(30,*,iostat=ios) comtid,compro(:),comtime(:)
	if(ios/=0) exit
	ncomt=ncomt+1
!	write(*,*) '# of records processed = ',i
	mfirst=mlast+1
	k=1
	j=j+1
	write(302,rec=ncomt) comtid,compro(:),comtime(:)

		do 
			read(30,*,iostat=ios) comtid2,compro(:),comtime(:)
			if(ios/=0) exit
			if(comtid(1:2)==comtid2(1:2)) then
			ncomt=ncomt+1
			k=k+1
			write(302,rec=ncomt)comtid2,compro(:),comtime(:)
			else
			backspace(30)
			exit
			end if
		end do

	mlast=mlast+k
	write(301,2011) j,comtid(1:2), mfirst,mlast

end do

!write(*,*) ncomt
! begin of commut frac code
j=0
nfrac=0
ffirst=0
flast=0
do 

	read(40,*,iostat=ios) fracid,(fracpro(:,ik),ik=1,ngroup)
	if(ios/=0) exit
	nfrac=nfrac+1
!	write(*,*) '# of records processed = ',i
	ffirst=flast+1
	k=1
	j=j+1
	write(402,rec=nfrac) fracid,(fracpro(:,ik),ik=1,ngroup)

		do 
			read(40,*,iostat=ios) fracid2,(fracpro(:,ik),ik=1,ngroup)
			if(ios/=0) exit
			if(fracid(1:2)==fracid2(1:2)) then
			nfrac=nfrac+1
			k=k+1
			write(402,rec=nfrac)fracid2,(fracpro(:,ik),ik=1,ngroup)
			else
			backspace(40)
			exit
			end if
		end do

	flast=flast+k
	write(401,2011) j,fracid(1:2), ffirst,flast

end do

!write(*,*) nfrac
! end of commut frac code
! read all states fips in two digits
!
do ii = 1, maxst
  read (98, 81) ij, stind(ii)
end do
81 format (i2, 6x, a, 6x, i2) 
!
! find out total number of population tracts in census file
ntract = 0
DO                                                     ! Loop for census tracts
  READ(14,*,IOSTAT=ios)                                ! Read population file
  IF (ios/=0) EXIT
  ntract = ntract + 1
END DO
!
rewind(14)
READ (14,*)                                         ! Discard header record
READ (14,*)                                         ! Skip header
!
ALLOCATE(pop(ngroup))                                      ! Dynamic allocation
!
! Read the census file and write population and tractid in direct access files
ntract = 0                                             ! initialisation

DO                                                     ! Loop for census tracts
  READ(14,*,IOSTAT=ios) tractid, pop                   ! Read population file
  IF (ios/=0) EXIT
  do ist = region1, region2
   if (stind(ist) == tractid(1:2)) then
      ntract=ntract+1                                  ! Increment tract count
      WRITE(24,REC=ntract) pop                         ! Write population file
      WRITE(44,rec=ntract) tractid,ntract              ! Write to indexing file
	  WRITE(441,*,IOSTAT=ios) tractid				   ! For state and county indexing purposes
     exit
   end if
  end do
END DO

REWIND(441)
!
! Assign first five digits (state+county) of census tractid
first=0
last =0
acount=0
Do

	read(441,*,IOSTAT=ios) tractid
	if(ios/=0) exit
	county1 = tractid(1:5)
	noinct=1
    acount=acount+1
	first=last+1

	do 
		read(441,*,IOSTAT=ios) tractid
		if(ios/=0) exit
			county2=tractid(1:5)
		if(county2==county1) then 
			noinct = noinct+1
		else 
			backspace(441)
			exit
		end if 
	end do 
    last=last+noinct
	write(96,*,IOSTAT=ios) county1, first, last

end do 

rewind(96)

!
sumcount = 0
Do 

	read(96,*,IOSTAT=ios) county1
	if(ios/=0) exit
	do jj = 1, maxst
      if (county1(1:2) == stind(jj)) then
        sumcount(jj) = sumcount(jj) + 1
        exit
      end if
    end do
end do 

stsump = 0
stsuma = 0
do ii = 1, maxst
  stsuma = stsump + sumcount(ii)
  write (97,*,IOSTAT=ios) stind(ii),stsump+1,stsuma
! write (51,80) ii,stind(ii),stsump+1,stsuma,sumcount(ii)
  stsump = stsuma
end do
!
!write total number of census tracts & counties in counter file
!
read (99,*)
write (99,*) ntract,acount
9991 format ('# of home tracts =',I7,' # of counties =',I7)
write (*,9991) ntract,acount
!
80 format (2x,i6,2x,a,2x,i6,2x,i6,2x,i6)
!
! Wrapup section
!
deallocate(distpro,areapro,fracpro)
CLOSE(14)                                              
CLOSE(24)                                              
close(44)
close(441,status='delete')
close(96)
close(97)
close(98)
close(99)
WRITE(18,*) 'total number of tracts is', ntract        ! # of population tract
DEALLOCATE(pop)                                        ! Free memory
CALL SYSTEM_CLOCK(cl2,cr,cm)                           ! Execution end time
WRITE(18,*) 'IndexPop End time  =',cl2                 ! Put time on log
WRITE(18,*) 'IndexPop Job time  =',REAL(cl2-cl1)/cr    ! Elapsed time
CLOSE(18)                                              ! Close log file
STOP                                                   ! Stop program
END PROGRAM indexpop7                                  ! End of source code
