CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C Subroutine Distribution
C INPUT  : 
C		UDist: 1 - Normal; 2 - Lognormal; 3 - Uniform
C		UPm1 = Mean for normal or lognormal or = minimum for uniform
C		UPm2 = Standard deviation for normal or lognormal or = maximum for uniform
C OUTPUT :
C		Pinvp - the random value selected from the specified distribution  
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      subroutine Distribution(UDist,UPm1,UPm2,UPm3,Pinvp,idum,ind,bl,bu)
      implicit none
      integer  I, ierr, cf, idum
	Integer ind				!ind=1,Pinvp>0;ind=2,0<Pinvp<1;ind=3,Pinvp>=1.0
      integer UDist, midpt, cnt
      real Pinvp, UPm1, UPm2,UPm3,bu, bl, mid, temp
	real lvar, meantemp
      
	midpt = 0 ! midpt = 0 for no midpoint calculation; midpt = 1 for midpoint
	cf =1 ! cf=1 to ensure a positive random value
	cnt = 0

	!bu=0 ! upper bound
	!bl=0 ! lower bound
	mid=0 
	temp=0

C	If the distribution is lognormal(2), the mean is a geometric mean
C     The geo-mean and std have to be converted.  
	if (UDist .eq. 2) then
		meantemp = UPm1 
		if (UPm2==0) then 
		lvar = 0
		else if (UPm2>0) then
		lvar = (log(UPm2))**2.0
		else
		write (*,*) ' standard deviation must be positive'
		stop
		end if 
		UPm1 = meantemp * exp(0.5*lvar)
		UPm2 = sqrt((meantemp**2.0)*exp(lvar)*(exp(lvar)-1.0))
C	write (*,*) UPm1, UPm2
	end if


C     If cf = 1 then the parameter must be positive.

C      if (cf .eq. 1) then
C       If the parameter must be positive then use BL and BU.
	if(bl==0 .AND.bu==0) then
	  if(ind==1) then
		BL = 0.0
		if (udist .eq. 2) BL = 1e-30 ! BL can not be zero if it is lognormal
C			Make the upper bound large enough that F(BU) = 1.0
          BU = 25.0*UPm2+UPm1
		if(UPm2==0) BU=1.2*UPm1
	  else if (ind==2) then 
		BL = 0.0
		if (udist .eq. 2) BL = 1e-30
          BU = 1
	  else if(ind==3) then 
	    BL = 0.9999999
		BU = 25.0*UPm2+UPm1
		if(UPm2==0) BU=1.2*UPm1
	  else 
		WRITE(*,*) 'ind has to be 1, or 2, or 3'
	  end if
	else
	  if (udist == 2 .AND. BL==0) BL = 1e-30
	  if (ind==3 .AND. BL<1) BL=0.9999999
	  if(UPm2==0) BU=1.2*UPm1
	end if

C        print *, 'BU=',BU(1),' Mean=',UPm1,' SD=',UPm2
C      end if
C     If the distribution is normal(1) or lognormal(2),
C     then the second parameter is the standard deviation.
C     The norml and lgnrml subroutines accept the variance.
      if (UDist .lt. 3) UPm2 = UPm2**2.0
C     If the dist is uniform(3), the parameters are the min and max.
C     The unifrm subroutine accepts the midpoint and length.
      If (UDist .eq. 3) then
        if (cf .eq. 1 .and. UPm1 .lt. bl) then
          temp = UPm2 - UPm1
          UPm1 = bl
        endif
        mid = (UPm2+UPm1)/2.0
        UPm2 = UPm2 - UPm1
        UPm1 = mid
        if (temp .gt. 0) then
          bl = (temp - UPm2)/temp  
        else
          bl = 0
        endif
      End If
      ierr = 0
	
      call parm(pinvp,UPm1,UPm2,Upm3,BU,BL,UDist,cf,ierr,midpt,idum)
      if (ierr .ne. 0) then
        print *, 'Error code ', ierr
        return
      end if
      return
      end




      subroutine parm(pinvp,pm1,pm2,pm3,bu,bl,dist,cf,ierr,midpt,idum)
c**********************************************************************
c
c variable           type                 description
c----------------------------------------------------------------------
c   pinvp           real			  A variable which contains a pvalue 
c								  in input and the value that produces the p-value
c                                   from the distribution on output
c
c  pm1            real*4			  The means of the 
c                                   distributions or the min for uniform
c
c  pm2            real*4			  variances of the 
c                                   distributions or the max for uniform
c  pm3			real			  peak at the most likely value for triangular
c								  distribution
c
c  bu               real*4		  upper bound on the parameter
c
c  bl               real*4	      lower bound on the parameters
c                                  
c
c  dist             integer		  distribution for each group
c                                   dist=1 -  normal
c                                   dist=2 -  log-normal
c                                   dist=3 -  uniform
c
c
c    cf            integer	     flag which indicates that the positive value 
c								 will be produced if cf=1
c
c
c     ierr           integer      error parameter
c                                  = 0  normal exit
c                                  = 9  p and/ or q is less than or equal 
c                                       to zero for the beta distribution
c                                  = 19 error in log normal distribution
c                                       specified gives bad values
c                                  
c**********************************************************************
c
      implicit real (a-h,o-z)
      implicit integer (i-n)
      integer cf,ierr,dist,midpt
      real pm1,pm2,pm3
      real pinvp,bu,bl

           if (midpt .eq. 1) then
             pinvp = 0.5
           else
             CALL RANDOM_SEED(idum)
             CALL RANDOM_NUMBER(pinvp)
           end if
c	Compute the random value (i.e., pinvp (input and output)) from the specified distribution
c	(compute the reverse normal distribution for the normally 
c	distributed parameters from the random 0-1 value in pinvp)
c
        if (dist .eq. 1) then
         call norml(pinvp,pm1,pm2,bu,bl,cf)
        else if (dist .eq. 2) then
         call lgnrml(pinvp,pm1,pm2,bu,bl,cf,ierr)
         if (ierr .eq. 19) then
              return
         endif
        else if (dist .eq. 3) then
         call unifrm(pinvp,pm1,pm2)
        else if (dist .eq. 4) then
	   call triangular(pinvp,pm1,pm2,pm3)
	  else
         print *, 'Invalid distribution ', dist
	   stop
        end if
      return
      end
c
c
c**********************************************************************
c  normal distribution 
c
c variable           type                 description
c----------------------------------------------------------------------
c     x1             real*8   value at which to evauate normal distrubtion
c**********************************************************************
c
      real*8 function phi(x1)
      implicit real*8 (a-h,o-z)
      implicit integer (i-n)
      data cont/.3989422804014327d0/
      x = x1
      x2 = dabs(x)
      if(x2 .gt. 8.2d0) then
         phi = 1.d0
         if(x .lt. 0.d0)phi = 0.d0
         return
      endif
      if(x2 .le. 1.4d0) then
         z = .5d0 * x * x
         sum1 = 1228010.107285365d0 + z * (158991.1678595134d0
     1     + z * (52023.84359183340d0 + z * (2446.553063231375d0
     2     + z * (299.2753946019257d0 + z * (3.750096883348302d0
     3     + z * .1245323303992320d0)))))
         sum2 = 1228010.107285365d0 + z * (568327.8702879685d0
     1    + z * (118665.4562926197d0 + z * (14407.25401972122d0
     2    + z * (1081.531752978052d0 + z * (48.06642591229388d0 + z)))))
         phi = .5d0 + x * cont * sum1 / sum2
      elseif(x2.le.3.d0) then
         z = .5d0 * x * x
         sum1 = 55292084240073.18d0 + z * (7964986114686.544d0 + z *
     1    (2718494907518.083d0 + z * (180919145380.0440d0 + z *
     2    (26916137172.19225d0 + z * (986714119.8272167d0 + z *
     3    (83804764.57237313d0 + z * (1645066.659270341d0 + z *
     4    (84438.30867506914d0 + z * (665.2755198214666d0 + z *
     5    (18.86183562269277d0 -.1309679527795864e-1*z))))))))))    
         sum2 = 55292084240073.18d0 + z * (26395680861377.60d0 + z *
     1    (5987846770636.633d0 + z * (853778178980.0466d0 + z *
     2    (85210906890.95287d0 + z * (6266028501.427893d0 + z *
     3    (347383264.0888559d0 + z * (14563954.88957962d0 + z *
     4    (454072.2767946769d0 + z * (10065.68249588229d0 + z *
     5    (143.4146851381486d0 + z))))))))))
         phi = .5d0 + x * cont * sum1 / sum2
      else
         ivan = 1
         kout = 19
         if(x2.lt.3.9d0)kout = 25
         if(x2.lt.3.5d0)kout = 31
         if(x.le.0.d0) then
            x = -x
            ivan = 2
         endif
         a1 = x
         a0 = 1.d0
         b1 = x*x+1.d0
         b0 = x
         con = 1.d0
         f1 = 0.d0
         do 10 i = 1,kout
            con = con+1.d0
            a2 = a1*x+a0*con
            b2 = b1*x+b0*con
            a0 = a1
            a1 = a2
            b0 = b1
            b1 = b2
   10    continue
         f2 = a2/b2
         phi = dexp(-.5d0 * x * x)*  cont * f2
         if(ivan.ne.2) phi = 1.d0 - phi
      endif  
      return
      end
c
c**********************************************************************
c  inverse normal distribution
c
c variable           type                 description
c----------------------------------------------------------------------
c     x1              real*8        probability to invert
c**********************************************************************
c
      real*8 function anorm(x1)
      implicit real*8 (a-h,o-z)
      implicit integer (i-n)
      x = x1
      if (x - .5d0 .lt. 0.0d0) then
         z = -1.0d0  
         x = 1.0d0-x
      elseif (x - .5d0 .eq. 0.0d0) then
         anorm = 0.0d0
         return
      else
         z = 1.0d0
      endif
   23 if (x - .78814461d0 .lt. 0.0d0) then
         x = 2.506628d0 * (x - .5d0)
         xa = x * x
         anorm = z * x * (1.0d0 + xa * (.1666667d0 + xa * (.05833333d0 
     1     + xa * (.02519841d0 + xa * (.01203979d0 + xa * 
     2     (.006103921d0 + xa * (.003217748d0 + xa * (.001743826d0
     3     +xa*(.0009647275d0 + xa * (.0005422926d0 + xa * 
     4     .0003087330d0))))))))))
      elseif (x - .78814461d0 .eq. 0.0d0) then
          anorm = 0.8d0 * z
      elseif (x - .8686431d0 .lt. 0.0d0) then
          x = (x - .8159399d0) * 3.758194d0
          anorm = z * (.9d0 + x * (1.0d0 + x * (.45d0 + x * (.4366667d0
     1      + x * (.447500d0 + x * (.5000533d0 + x * (.5872400d0
     2      + x * (.7134978d0 + x * 8.8876785d0))))))))                                  
      elseif (x - .8686431d0 .eq. 0.0d0) then
          anorm = 1.12d0 * z
      elseif(x - .9278550d0 .lt. 0.0d0) then
          x = (x - .8925123d0) * 5.407239d0
          anorm = z * (1.24d0 + x * (1.0d0 + x * (.62d0 + x * 
     1       (.6792000d0 + x * (.8383227d0 + x * (1.120589d0 + x *
     2       (1.570603d0 + x * (2.274347d0 + x * (3.372258d0 + x * 
     3        5.090805d0)))))))))
      elseif(x - .9278550d0 .eq. 0.0d0) then
          anorm = 1.46d0 * z
      elseif (x - .9712834d0 .lt. 0.0d0) then
          x = 9.618983d0 * (x - .9494974d0)
          anorm = z * (1.64d0 + x * (1.0d0 + x * (.82d0 + x * 
     1       (1.063200d0 + x * (1.581069d0 + x * (2.536136d0 + x *
     2       (4.263734d0 + x * (7.401885d0 + x * (13.15285d0 + x *
     3       (23.78969d0 + x * (43.63140d0 + x * 80.92452d0)))))))))))
      elseif (x - .9712834d0 .eq. 0.0d0) then
          anorm = 1.90d0 * z
      elseif (x - .9860966d0 .lt. 0.0d0) then
          x = 19.28135d0 * (x - .9783083d0)
           anorm = z * (2.02d0 + x * (1.0d0 + x * (1.01d0 + x * 
     1        (1.5268d0 + x * (2.649769d0 + x * (4.952419d0 + x
     2        * (9.693672d0 + x * (19.58299d0 + x * (40.48079d0
     3        + 85.15398d0 * x)))))))))
      elseif (x - .9860966d0 .eq. 0.0d0) then
          anorm = 2.2d0 * z
      elseif (x - .9908625d0 .lt. 0.0d0) then
          x = 33.72160d0 * (x - .9886961d0)
          anorm = z * (2.28d0 + x * (1.0d0 + x * (1.14d0 + x * 
     1       (1.899467d0 + x * (3.628088d0 + x * (7.455716d0 + x
     2       * (16.03753d0 + x * (35.39291d0 + x * (80.81148d0 + 
     3        186.6810d0 * x)))))))))
      elseif (x - .9908625d0 .eq. 0.0d0) then
          anorm = 2.36d0 * z
      elseif (x - .9937903d0 .lt. 0.0d0) then
          x = 46.85879d0 * (x - .9922397d0)
          anorm = z * (2.42d0 + x * (1.0d0 + x * (1.21d0 + x * 
     1      (2.118800d0 + x * (4.248955d0 + x * (9.162771d0 + x * 
     2      (20.67714d0 + x * 48.13495d0)))))))
      elseif (x - .9937903d0 .eq. 0.0d0) then
          anorm  = 2.5d0 * z
      elseif (x - .9980166d0 .lt. 0.0d0) then
          x = 81.75595d0 * (x - .9958547d0)
          anorm = z * (2.64d0 + x * (1.0d0 + x * (1.32d0 + x * 
     1      (2.489867d0 + x * (5.369936d0 + x * (12.44508d0 + x
     2      *(30.1698d0 + x * (75.43037d0 + x * (192.8775d0 + x *
     3      (501.7034d0 + x * (1322.689d0 + x * (3525.225d0 + x *
     4      (9479.936d0 + x * (25685.32d0 + 70038.70d0 
     5       * x))))))))))))))
      elseif (x - .9980166d0 .eq. 0.0d0) then
          anorm = 2.88d0 * z
      elseif (x - .9993129d0 .lt. 0.0d0) then
          x = 225.6395d0 * (x - .9986501d0)
          anorm = z * (3.0d0 + x * (1.0d0 + x * (1.5d0 + x * 
     1      (3.166667d0 + x * (7.625000d0 + x * (19.70833d0 + x  
     2      * (53.25417d0 + x * (148.3538d0 + x * (422.5697d0 + x  
     3      * (1224.197d0 + x * (3594.111d0 + x * (10666.10d0 + x 
     4      * (31935.45d0 + x * 96332.42d0)))))))))))))
      elseif (x - .9993129d0 .eq. 0.0d0) then
          anorm = 3.2d0 * z
      elseif (x - .9997674d0 .lt. 0.0d0) then
          x = 580.5283d0 * (x - .9995166d0)
          anorm = z * (3.3d0 + x * (1.0d0 + x * (1.65d0 + x * 
     1       (3.796667d0 + x * (9.946750d0 + x * (27.95125d0 + x * 
     2       (82.07921d0 + x * (248.4234d0 + x * (768.6508d0 + x * 
     3       (2418.593d0 + x * (7711.549d0 + x * (24852.01d0 + x *  
     4       (80799.51d0 + x * (264646.7d0 + 872279.6d0 * 
     5        x ))))))))))))))
      elseif (x - .9997674d0 .eq. 0.0d0) then
          anorm = 3.5d0 * z
      elseif (x - .9999116d0 .lt. 0.0d0) then
          x = 1634.249d0 * (x-.9998409d0)
          anorm = z * (3.6d0 + x * (1.0d0 + x * (1.8d0 + x * 
     1       (4.486667d0 + x * (12.71400d0 + x * (38.61865d0 + x *
     2       (122.5368d0 + x * (400.6489d0 + x * (1338.971d0 + x *
     3       (4550.155d0 + x * (15667.13d0 + 54521.14d0 * x)))))))))))
      elseif (x - .9999116d0 .eq. 0.0d0) then
          anorm = 3.75d0 * z
      elseif (x - .9999744d0 .lt. 0.0d0) then
          x = 4147.199d0 * (x - .9999409d0)
          anorm = z * (3.85d0 + x * (1.0d0 + x * (1.925d0 + x * 
     1       (5.107500d0 + x * (15.38957d0 + x * (49.68159d0 + x * 
     2       (167.4959d0 + x * (581.7936d0 + x * (2065.349d0 + x * 
     3       (7454.696d0 + x * (27261.35d0 + x * (100752.4d0 + x * 
     4       (375621.8d0 + x * (1410664.d0 + 5330905.0d0 * 
     5        x ))))))))))))))
      elseif (x - .9999744d0 .eq. 0.0d0) then
          anorm = 4.05d0 * z
      elseif (x - .9999915d0 .lt. 0.0d0) then
          x = 13769.19d0 * (x - .9999834)
          anorm = z * (4.15d0 + x * (1.0d0 + x * (2.075d0 + x * 
     1      (5.907500d0 + x * (19.07876d0 + x * (65.98319d0 + x * 
     2      (238.2519d0 + x * (886.1746d0 + x * (3368.291d0 + x *  
     3      (13015.89d0 + x * (50955.36d0 + x * (201592.4d0 + 
     4      804502.4d0 * x))))))))))))
      elseif (x - .9999915d0 .eq. 0.0d0) then
          anorm = 4.3d0 * z
      elseif (x - .9999973d0 .lt. 0.0d0) then
          x = 40092.26d0 * (x - .9999946d0)
          anorm = z * (4.4d0 + x * (1.0d0 + x * (2.2d0 + x * (6.620000d0
     1     + x * (22.57933d0 + x * (82.44159d0 + x * (314.2059d0 + x * 
     2     (1233.407d0 + x * (4947.294d0 + x * (20173.20d0 + x * 
     3     (83332.14d0 + x * (347857.8d0 + x * (1464693.0d0 + x * 
     4     (6212166.0d0 + 26510760.0d0 * x)))))))))))))) 
      elseif (x - .9999973d0 .eq. 0.0d0) then
          anorm = 4.55d0 * z
      elseif(x - .9999992d0 .lt. 0.0d0) then
          x = 124267.1d0 * (x - .9999983d0)
          anorm = z * (4.65d0 + x * (1.0d0 + x * (2.325d0 + x * 
     1       (7.374167d0 + x * (26.49241d0 + x * (101.8535d0 + x *  
     2       (408.6823d0 + x * (1688.770d0 + x * (7130.013d0 + x * 
     3       (30600.67d0 + x * (133040.3d0 + x * (584483.0d0 + 
     4       2590026.0d0 * x))))))))))))
      elseif(x - .9999992d0 .eq. 0.0d0) then
          anorm = 4.8d0 * z
      else
          x = 410010.7d0 * (x - .9999995d0)
          anorm = z * (4.9d0 + x * (1.0d0 + x * (2.45d0 + x * 
     1      (8.170000d0 + x * (30.84142d0 + x * (124.5582d0 + x * 
     2      (524.9252d0 + x * (2277.999d0 + x * (10099.83d0 + x * 
     3      (45516.93d0 + x * (207791.0d0 + x * (958525.1d0 +
     4       4459776.0d0 * x))))))))))))
      endif
      return
      end
c
c**********************************************************************
c
c  subroutine for computing the inverse log normal distribution 
c
c
c variable           type                 description
c----------------------------------------------------------------------
c  mean             real*4	         means of the 
c                                      normal distribution of the
c                                      log-normal variables
c
c var               real*4	         variances of the 
c                                      normal distribution for the
c                                      log-normal variables
c
c bu                real*4            upper bound on the lognormal variable
c                                    
c
c bl                real*4            lower bound on the lognormal variable
c
c tflag            integer          flag which ensures that the random value selected
c                                   from distribution is positive
c
c   pinvp             real*4          the variable which contains
c                                     p-values on input and the value which
c                                     produces that p-value from the 
c                                     distribution on output
c
c**********************************************************************

      subroutine lgnrml(pinvp,mean,var,bu,bl,tflag,ierr)
      implicit real*4 (a-h,o-z)
      implicit integer (i-n)
      real*8 phi, anorm
      integer   first,tflag
      real * 4      mean, var,lxsim,lmean
      real * 4      pinvp
        sdl = sqrt(log(var / (mean**2) + 1.0))
        tmp = mean 
        if (tmp .lt. 0) then
           ierr = 19
        endif
        lmean = log(tmp) - .5 * sdl**2
        p1 = 0
             fl = (log(bl) - lmean) / sdl
             p1 = sngl(phi(dble(fl)))
             fu = (log(bu) - lmean) / sdl
             p2 = sngl(phi(dble(fu)))
        if (p2 .eq. p1) then
            pinvp = -9999
        else
 
             if (tflag .eq. 1) then
                  p = pinvp * (p2 - p1) + p1
               else
                  p = pinvp
              endif
              ap = sngl(anorm(dble(p)))
              lxsim = ap * sdl + lmean
              pinvp = exp(lxsim)   
        endif
      return
      end

c
c**********************************************************************
c
c  subroutine for computing the inverse normal distribution with or
c      without a threshold on the group
c
c
c variable           type                 description
c----------------------------------------------------------------------
c  mean             real*4             the means of the 
c                                      normal parameters
c
c var               real*4            the variances of the 
c                                      normal parameters
c
c bu                real*4            upper bound on the normal parameter
c           
c
c bl                real*4            lower bound on the normal parameter
c                                     
c
c tflag            integer          tflag = 1 to ensure a positive value
C                                    
c
c   pinvp             real*4          the variable which contains
c                                     p-values on input and the value which
c                                     produces that p-value from the 
c                                     distribution on output
c
c**********************************************************************

      subroutine norml(pinvp,mean,var,bu,bl,tflag)
      implicit real*4 (a-h,o-z)
      implicit integer (i-n)
      real*8 phi, anorm
      integer  tflag
      real * 4      mean, var, nmean
      real * 4      pinvp

        sdn = sqrt(var)
        nmean = mean
        p1 = 0
c        write(19,*) ' mean = ',nmean
c        write(19,*) ' bu = ',bu
c        write(19,*) ' bl = ',bl
             fl = (bl - nmean) / sdn
             p1 = sngl(phi(dble(fl)))
             fu = (bu - nmean) / sdn
             p2 = sngl(phi(dble(fu)))
        if (p2 .eq. p1) then
            pinvp = -9999
        else
        
             if (tflag .eq. 1) then
                  p = pinvp * (p2 - p1) + p1
             else
                  p = pinvp
             endif
              ap = sngl(anorm(dble(p)))
              pinvp = ap * sdn + nmean  
	  end if 
      return
      end
c      
c**********************************************************************
c
c  subroutine for computing the inverse uniform distribution
c
c variable           type                 description
c----------------------------------------------------------------------
c
c   mid             real*4		    mid = (pm1 + pm2)/2
c                                     
c
c   spread            real*4          spread = pm2 - pm1
c
c   pinvp             real*4		    the varibale which contains
c                                     p-values on input and the value which
c                                     produces that p-value from the 
c                                     distribution on output
c
c**********************************************************************

      subroutine unifrm(pinvp,mid,spread)
      implicit real (a-h,o-z)
      implicit integer (i-n)
      integer   first
      real mid
	real spread
	real bottom,pinvp
        bottom = mid - spread/2.0e0
          p = pinvp
C	write(*,*) bottom, mid, spread, p
          pinvp = bottom + p * spread
C	write (*,*) pinvp

      return
      end
c**********************************************************************
c  subroutine for computing the inverse triangular distribution
c
c variable           type                 description
c----------------------------------------------------------------------
c	min				real			minimum (pm1) 
c     max				real			maximum (pm2)
c	peak			real			the most likely value
c
c	pinvp			real*4		    the varibale which contains
c                                     p-values on input and the value which
c                                     produces that p-value from the 
c                                     distribution on output
c
c**********************************************************************

	subroutine triangular(pinvp,min,max,peak)
	real min,max,peak
	real pinvp, Pmid

	Pmid = (peak-min)/(max-min)

	if (pinvp .le. Pmid) then 

		pinvp = sqrt(pinvp * (peak-min) * (max-min)) + min
	
	else
	
		pinvp = max - sqrt((1-pinvp) * (max - peak) * (max-min))
	
	end if
	return
	end 
